#include <afxwin.h>
#include "CUserScreen.h"
#include "Constants.h"
#include "ResourceManager.h"
#include "PhotocellTextBoxes.h"
#include "MyToolTip.h"
#include "stdafx.h"
//#include "TrackList.h"


/*************************************
  Additions by bmcvey 6-26-01
    
	  - altered the menu (so menu.rc and constants.h, added afxres.h to menu.rc)
	  - although not this file, added StdAfx.cpp and StdAfx.h to facilitate
	     the dialog window and message(data) passing to parent window,
		 specifically TRAINPROP struct
	  - added functions OnAddTrain, OnAddTrainFromFile
	  - removed code to read traindata from file from constructor
	  - replaced myNEDlg with myATDlg, deleted myTPDlg
	  - deletes myATDlg from destructor if necessary
	  - added functions OnApply and OnDialogDestroy
	  - added to message maps for the 4 function mentioned above
***************************************/
/////////////////////////////////////////////////////////////////////
//Constructor for CUserScreen.  Creates the window which is ajusted in OnCreate
//creates a resourcemanger to handle operating system and creates TrainControls 
//to allow the user to control the trains. Init data structures and sets a timer
//to update visual
/////////////////////////////////////////////////////////////////////////////////
CUserScreen::CUserScreen()
{

	CRect Rect(0,0,500, 500);  //define oringal window

	Create(0, "User Window", WS_OVERLAPPEDWINDOW, Rect, NULL, "CUSERSCREENMENU");  //create new window


 	CRect MyRect(UpperLeftScreenX, UpperLeftScreenY, 
		UpperLeftScreenX+ m_cxChar*50 + BitWidth, 
		UpperLeftScreenY+FindMax(m_cyChar * 25, BitHeight + SPACER)); 
	// comes back with m_cxChar and my_cyChar defined set up new rect for new window

	CalcWindowRect(&MyRect);  // calc pixel width and height for window

	SetWindowPos(NULL, UpperLeftScreenX, UpperLeftScreenY, MyRect.Width() , MyRect.Height(), 
		         SWP_NOZORDER | SWP_NOMOVE | SWP_NOREDRAW);

	MyResourceManager = new ResourceManager;
	//create resourcemanager to act as operating system

	int i;

	//init data sturctures to NULL
	for (i = 0; i<MAXTRAINS; i++)
	{
		AllTrains[i] = NULL; 
		TrainControls[i] = NULL;
	}
    myATDlg = NULL;

	NumTrains = 0; //init numTrains to 0
	

	LoadTurnouts();  //load position and color of turnouts
	LoadPhotocells(); //load positon and color of photocells

	//init previous and current status of photocells and turnouts to zero
	for (i = 0; i<MAXPHOTOCELLS; i++)
	{
		PrevPC[i] = 0;  
		CurPC[i] = 0;
	}

	for (i = 0; i < MAXTURNOUTS; i++)
	{
		PrevTO[i] = 0;
		CurTO[i] = 0;
	}


 
	SetTimer( 3, 500, NULL ); // set timer for every second

}

///////////////////////////////////////////////////////////////////
//Deletes the windows created.  The resourcemanager and the trains pointers
//////////////////////////////////////////////////////////////////////////

CUserScreen::~CUserScreen(){

//MessageBox("CUSERSCREEN", "destructor");
	for (int i = 0; i<MAXTRAINS; i++)
	{
		if (TrainControls[i] != NULL)
			delete TrainControls[i];
		if (AllTrains[i] != NULL)
			delete AllTrains[i];

		//Delete the trains/traincontrols that are not null because we created them
	}
	if (myATDlg)
		delete myATDlg;
	delete MyResourceManager; 
	//not needing the resourcemanger if the user can't see the visual anymore
}

////////////////////////////////////////////////////////////////////////
//Calls when user exits from menu -- send close  SendMessage( WM_CLOSE )
/////////////////////////////////////////////////////////////////////
void CUserScreen::OnExit()
{
	 SendMessage( WM_CLOSE ); 
}

////////////////////////////////////////////////////////////////////////
//Creates layout of screen in proportion to the chosen font.  Creates all 
//Buttons, Labels, ListBoxes and inputs the bitmap. 
//////////////////////////////////////////////////////////////////////
int CUserScreen::OnCreate(LPCREATESTRUCT lpcs){


	if (CWnd::OnCreate(lpcs) == -1)
		return -1; //return if not good


	m_font.CreatePointFont(80, _T("MS Sans Serif"));  //select our font
	CClientDC dc (this);  //get handler to compute text averages
	CFont * pOldFont = dc.SelectObject(&m_font); // select our font

	TEXTMETRIC Tm; // get Textmetric

	dc.GetTextMetrics(&Tm);  // and give it our font to calculate
	m_cxChar = Tm.tmAveCharWidth;  // store average width
	m_cyChar = Tm.tmHeight + Tm.tmExternalLeading;	// and height
	
    TrackLayoutBitmap.LoadBitmap( "IDB_TRACKLAYOUT");  // load the bitmap
    m_memDC.CreateCompatibleDC( &dc );			// Create the compatible in memory
    m_memDC.SelectObject( &TrackLayoutBitmap);	

	BITMAP b; 

	TrackLayoutBitmap.GetBitmap(&b); //this functions makes b have our properties

	BitHeight = b.bmHeight;  //set our bitmaps height and width
	BitWidth = b.bmWidth;

	UpperLeftBitmap.x = m_cxChar*50;  
	//find where to put the bitmap in relation to out font
	UpperLeftBitmap.y = (int)(m_cyChar * .5);
	//find wher the bitmaps lower corner is

    LowerRightBitmap.x = UpperLeftBitmap.x + BitWidth;
    LowerRightBitmap.y = FindMax(m_cyChar * 25, BitHeight);


	//loads location of the CStatics that shows allocation above photocells
	LoadPhotocellTextBoxes();

	CRect rect;


	//Create Course controls
	rect.SetRect (UpperLeftScreenX + m_cxChar * 1, 
		          UpperLeftScreenY + m_cyChar * 1, 
				  UpperLeftScreenX + m_cxChar * 24, 
				  UpperLeftScreenY + m_cyChar * 4);
	
	dc.FillSolidRect(rect,RGB(255, 0, 0));
	CourseControlGroupBox.Create (_T ("Course Controls"),  WS_CHILD | WS_VISIBLE | BS_GROUPBOX,
        rect, this, (UINT) -1);

		rect.SetRect (UpperLeftScreenX + m_cxChar * 4, 
			         UpperLeftScreenY + (int)(m_cyChar * 2), 
					 UpperLeftScreenX + m_cxChar * 22, 
					 UpperLeftScreenY + (int)(m_cyChar * 3.5));

//		EnableTrackToggleButton.Create (_T ("Disable Track"), WS_CHILD | WS_VISIBLE |
//				BS_PUSHBUTTON, rect, this, IDC_ENABLETRACKBUTTON);
	redbutton.LoadBitmap(IDS_RED_BUTTON);    // bmcvey
	greenbutton.LoadBitmap(IDS_GREEN_BUTTON);  //bmcvey
		EnableTrackToggleButton.Create (_T ("Disable Track"), WS_CHILD | WS_VISIBLE |
				BS_BITMAP, rect, this, IDC_ENABLETRACKBUTTON);   // bmcvey - colored buttons
		EnableTrackToggleButton.SetBitmap(redbutton);     // bmcvey

		//Creates the User Message List Box that informs the user about turnouts
	rect.SetRect (UpperLeftScreenX+ m_cxChar * 1, 
				 UpperLeftScreenY + (int)(m_cyChar * 20),
				 UpperLeftScreenX +m_cxChar * 45, 
				 UpperLeftScreenY + m_cyChar * 30);
	UserMessages.Create (WS_CHILD |WS_BORDER| WS_VSCROLL|WS_HSCROLL| WS_VISIBLE, rect, this, IDC_MESSAGELB);

	rect.SetRect (UpperLeftScreenX + m_cxChar * 1, 
				  UpperLeftScreenY + (int)(m_cyChar * 19),
				  UpperLeftScreenX + m_cxChar * 45, 
				  UpperLeftScreenY + (int)(m_cyChar * 20));

	MessageText.Create (_T ("Report"), WS_CHILD | WS_VISIBLE | SS_LEFT, rect,
		         this, IDC_MESSAGETEXT);




	//Create the Listbox that outputs all train statuses
	rect.SetRect (UpperLeftScreenX + m_cxChar * 1, 
				 UpperLeftScreenY + (int)(m_cyChar * 5),
				 UpperLeftScreenX + m_cxChar * 45, 
				 UpperLeftScreenY + (int)(m_cyChar * 6));

	TrainInfoText.Create (_T ("Train Information"), WS_CHILD | WS_VISIBLE | SS_LEFT, rect,
		         this, IDC_TRAININFOTEXT);


	rect.SetRect (UpperLeftScreenX+ m_cxChar * 1, 
				 UpperLeftScreenY + (int)(m_cyChar * 6),
				 UpperLeftScreenX +m_cxChar * 45, 
				 UpperLeftScreenY + m_cyChar * 18);
	TrainInfo.Create (WS_CHILD |WS_BORDER| WS_VSCROLL|WS_HSCROLL| WS_VISIBLE |LBS_STANDARD, rect, this, IDC_TRAININFOLB);



	//Create the CStatic part of the PhotocellOwnership
	//Shows allocation of photocells
	for (int i= 0; i<MAXPHOTOCELLS; i++)
	{
		rect.SetRect (PhotocellOwnership[i].Location.x, 
					PhotocellOwnership[i].Location.y,
					PhotocellOwnership[i].Location.x + 2*m_cxChar, 
					PhotocellOwnership[i].Location.y + 1*m_cyChar);

		PhotocellOwnership[i].Message.Create(_T (""), WS_CHILD | WS_VISIBLE | SS_LEFT, rect,
		         this, IDC_PHOTOCELLBASE + i);

		PhotocellOwnership[i].Message.SetFont(&m_font, FALSE);

	}


	//Set out font to all of the buttons on screen
	EnableTrackToggleButton.SetFont (&m_font, FALSE);
	CourseControlGroupBox.SetFont (&m_font, FALSE);
	TrainInfoText.SetFont (&m_font, FALSE);
	MessageText.SetFont (&m_font, FALSE);
	UserMessages.SetFont (&m_font, FALSE);
	TrainInfo.SetFont (&m_font, FALSE);

	//Set the background color to gray
	WindowBackgroundColor = ::GetSysColor (COLOR_3DFACE);
	
	//Make our Tool Tips the items that flash up on screen when mouse is over
	//a button
	//Couldn't get this to work for CStatics
	m_ctlTT.Create (this);
   	m_ctlTT.AddWindowTool (GetDlgItem (IDC_ENABLETRACKBUTTON),
        MAKEINTRESOURCE (IDS_ENABLETRACKBUTTON));

	dc.SelectObject(pOldFont); // return old pen to not consume resources
	
	return 0;

}//Oncreate


/////////////////////////////////////////////////////////////////////////
//Repainst the screen--Called when user covers or shrinks the window
//Also called by OnTimer to keep the display correct
///////////////////////////////////////////////////////////////////////
afx_msg void CUserScreen::OnPaint(){


  CRect rect;
  GetClientRect(&rect);
  CPaintDC dc(this);
  dc.FillSolidRect(rect, WindowBackgroundColor);
  //repatin the screen with the background color

  //show the bitmap on the screen
  dc.BitBlt(UpperLeftBitmap.x,UpperLeftBitmap.y , LowerRightBitmap.x, 
	        LowerRightBitmap.y , &m_memDC, 0, 0, SRCCOPY );

 
  //define our brushes to paint the photocells and turnouts
  CBrush ColorRedBrush;  //turnout brushes
  CBrush ColorGreenBrush;

  CBrush ColorBlueBrush; //photocell brushes
  CBrush ColorWhiteBrush;

  CBrush * pbrush;

  //set the color of brushes
  ColorRedBrush.CreateSolidBrush(RGB(255,0, 0));  //red
  ColorBlueBrush.CreateSolidBrush(RGB(0,0, 255)); //blue
  ColorWhiteBrush.CreateSolidBrush(RGB(255,255, 255)); //white
  ColorGreenBrush.CreateSolidBrush(RGB(0,255, 0)); //green


  //Repaints all the turnouts
  for (int i = 0; i<MAXTURNOUTS ; i++)
  {
	 
	  if (OutsideTurnouts[i].Color == RED)
		  pbrush = dc.SelectObject(&ColorRedBrush);
	  else 
		  pbrush = dc.SelectObject(&ColorGreenBrush);
  
	  //redraw the elipse with current color
	  dc.Ellipse(OutsideTurnouts[i].Location.x - radius, OutsideTurnouts[i].Location.y -radius,
	  OutsideTurnouts[i].Location.x + radius, OutsideTurnouts[i].Location.y + radius);
	  dc.SelectObject(pbrush);

  }


   for (i = 0; i<MAXTURNOUTS ; i++)
  {
	 
	  if (InsideTurnouts[i].Color == RED)
		  pbrush = dc.SelectObject(&ColorRedBrush);
	  else 
		  pbrush = dc.SelectObject(&ColorGreenBrush);
  
	  dc.Ellipse(InsideTurnouts[i].Location.x - radius, InsideTurnouts[i].Location.y -radius,
	  InsideTurnouts[i].Location.x + radius, InsideTurnouts[i].Location.y + radius);
	  dc.SelectObject(pbrush);

  }

   char temp[10];

   //repaint the photocells with their current color
   for (i = 0; i<MAXPHOTOCELLS ; i++)
   {
	 
	  if (PhotocellPoints[i].Color == WHITE)
		  pbrush = dc.SelectObject(&ColorWhiteBrush);
	  else 
		  pbrush = dc.SelectObject(&ColorBlueBrush);
  
	  dc.Ellipse(PhotocellPoints[i].Location.x - PCradius, PhotocellPoints[i].Location.y -PCradius,
	  PhotocellPoints[i].Location.x + PCradius, PhotocellPoints[i].Location.y + PCradius);
	  dc.SelectObject(pbrush);

	  int owned = MyResourceManager->GetPhotoCellManager(i);
	  if (owned != 0)
		itoa(owned, temp, 10);
	  else
		  strcpy(temp, "N");

	  //and update the contents of the ownership CStatic to show
	  //user which photocells are owned by which Trains
	  PhotocellOwnership[i].Message.SetWindowText(temp);


   }

}//OnPaint


///////////////////////////////////////////////////////////////////
//Called when window is destroyed kills the timer
/////////////////////////////////////////////////////////////////
afx_msg void CUserScreen::OnDestroy()
{
//MessageBox("CUserScreen", "OnDestroy");
	KillTimer( 3 );   //kill timer 3
	delete this;

}

/////////////////////////////////////////////////////////////////////////
//Called when timer is fired.  Cycles through turnouts and photocells to
//check if there was a change since last timer.  If there is we invalide the 
//rect where the change occurred to force a call to OnPaint
///////////////////////////////////////////////////////////////////////////
afx_msg void CUserScreen::OnTimer()
{
	int i;
	CRect rect;

	//update status of turnouts
	for(i = 0; i<MAXTURNOUTS; i++)
		CurTO[i] = MyResourceManager->GetTurnoutDirection(i);

	for(i = 0; i<MAXTURNOUTS; i++)
	{
		 if (CurTO[i] < PrevTO[i]) //was curved not straght
		 {

			 //so set color to be what it should be when turnout is straight
			 InsideTurnouts[i].Color = InsideTurnouts[i].ColorWhenStraight;
			 OutsideTurnouts[i].Color = OutsideTurnouts[i].ColorWhenStraight;

			 rect.SetRect(InsideTurnouts[i].Location.x-radius, 
			         InsideTurnouts[i].Location.y-radius, 
					 InsideTurnouts[i].Location.x+radius, 
					 InsideTurnouts[i].Location.y +radius);

			 // and force it to be updated by OnPaint by invalidating it
			InvalidateRect(rect,FALSE);

			rect.SetRect(OutsideTurnouts[i].Location.x-radius, 
			         OutsideTurnouts[i].Location.y-radius, 
					 OutsideTurnouts[i].Location.x+radius, 
					 OutsideTurnouts[i].Location.y +radius);

			InvalidateRect(rect,FALSE);

			 
		 }
		 else if (CurTO[i] > PrevTO[i]) //was straight now curved
		 {
			 //adjust color to show curved
		    InsideTurnouts[i].Color = InsideTurnouts[i].ColorWhenCurved;
		   	OutsideTurnouts[i].Color = OutsideTurnouts[i].ColorWhenCurved;

			rect.SetRect(InsideTurnouts[i].Location.x-radius, 
			         InsideTurnouts[i].Location.y-radius, 
					 InsideTurnouts[i].Location.x+radius, 
					 InsideTurnouts[i].Location.y +radius);

			//and force it to be redrawn by Onpaint by invalidating the rect
			InvalidateRect(rect,FALSE);

			rect.SetRect(OutsideTurnouts[i].Location.x-radius, 
			         OutsideTurnouts[i].Location.y-radius, 
					 OutsideTurnouts[i].Location.x+radius, 
					 OutsideTurnouts[i].Location.y +radius);

			InvalidateRect(rect,FALSE);

		 }


		 PrevTO[i] = CurTO[i]; //update the prev for next timer 

	}


	//update the status of all photocells
	for (i = 0; i<MAXPHOTOCELLS; i++)
		CurPC[i] = MyResourceManager->GetPhotoCellStatus(i);


	for (i = 0; i<MAXPHOTOCELLS; i++)
	{

		if (CurPC[i] > PrevPC[i] ) //was uncovered now covered
		{
			//so adjust color to blue
			PhotocellPoints[i].Color = BLUE;
			rect.SetRect(PhotocellPoints[i].Location.x-PCradius, 
			         PhotocellPoints[i].Location.y-PCradius, 
					 PhotocellPoints[i].Location.x+PCradius, 
					 PhotocellPoints[i].Location.y +PCradius);
			//and paint it in OnPaint
			InvalidateRect(rect,FALSE);

			//Change Ownership if necessary since it was covered it might need
			//to be updated
			rect.SetRect (PhotocellOwnership[i].Location.x, 
					PhotocellOwnership[i].Location.y,
					PhotocellOwnership[i].Location.x + 2*m_cxChar, 
					PhotocellOwnership[i].Location.y + 1*m_cyChar);

			InvalidateRect(rect,FALSE);
		}
		else if  (CurPC[i] < PrevPC[i]) //was covered now uncoverd
		{

			//adjust color to white
			PhotocellPoints[i].Color = WHITE;
			rect.SetRect(PhotocellPoints[i].Location.x-PCradius, 
			         PhotocellPoints[i].Location.y-PCradius, 
					 PhotocellPoints[i].Location.x+PCradius, 
					 PhotocellPoints[i].Location.y +PCradius);
			//and paint in by forceing a call to OnPaint
			InvalidateRect(rect,FALSE);

			rect.SetRect (PhotocellOwnership[i].Location.x, 
					PhotocellOwnership[i].Location.y,
					PhotocellOwnership[i].Location.x + 2*m_cxChar, 
					PhotocellOwnership[i].Location.y + 1*m_cyChar);

			InvalidateRect(rect,FALSE);
	
		
		}

		PrevPC[i] = CurPC[i];  //set the prev for next timer fire

	}

	TrainInfo.ResetContent(); //clear the Listbox with all trains in it
	char message[45];
	char temp[10];
	
	
	//And output current data for all the trains
	
	for (i = 0; i<NumTrains; i++)
	{
		char S[10] = "Speed: ";
		strcpy(message, AllTrains[i]->GetName());
		if (AllTrains[i]->GetDirectionAroundTrack() == CLOCKWISE)
			strcat(message, " Dir: CW ");
		else
			strcat(message, " Dir: CC ");

        if (AllTrains[i]->GetBlockedStatus() == ACTIVE)
		    strcat(message, "Status: Active ");
		else
			strcat(message, "Status: Blocked ");

		int speed = AllTrains[i]->GetSpeed();
		itoa(speed, temp, 10);
		strcat(S, temp);
		strcat(message, S);
		TrainInfo.AddString(message);
	}
}

////////////////////////////////////////////////////////////////////////
//Pressed the Enabled/Disabled CourseButton--Calls function with resourcemanger
//to handle it and updates the screen if the resourcemanger was successful
/////////////////////////////////////////////////////////////////////////
afx_msg void CUserScreen::OnToggleEnabledCourse()
{

		MyResourceManager->ToggleEnabledCourse();
/*
		if (MyResourceManager->GetTrackEnabled())
			EnableTrackToggleButton.SetWindowText("Disable Track");
		else
			EnableTrackToggleButton.SetWindowText("Enable Track");
*/
// bmcvey
		if (MyResourceManager->GetTrackEnabled())
			EnableTrackToggleButton.SetBitmap(redbutton);
		else
			EnableTrackToggleButton.SetBitmap(greenbutton);
}


//////////////////////////////////////////////////////////////////////
//Traps the clicking of Left Mouse Button and checks if the click was
//over a turnout.  If it was it calls the resource manger to toggle the
//turnout if possible
/////////////////////////////////////////////////////////////////////
afx_msg void CUserScreen::OnLButtonDown( UINT uFlags, CPoint point )
{

	//uncomment this to simply click on screen and get x,y cordinates
	//to place photocells and turnouts
/*********************************************************************	
     char temp[50];
	 CClientDC dc(this);

     int x = point.x - UpperLeftBitmap.x;
	 int y = point.y - UpperLeftBitmap.y;

	 itoa(x, temp, 10);
	 MessageBox(temp);

	 itoa(y, temp, 10);
	 MessageBox(temp);
***********************************************************************/
	

	for (int i = 0; i<MAXTURNOUTS; i++)
	{
	
		//check if the click was on the inside turnouts
		if (abs(point.x - InsideTurnouts[i].Location.x) < radius &&
			abs(point.y - InsideTurnouts[i].Location.y) < radius)

		{
			//if it is red try to toggle it with call to resource manager
			if (InsideTurnouts[i].Color == RED)
			{
				if (!MyResourceManager->ToggleTurnout(i))
    				UserMessages.AddString("Cannot toggle Train is Blocking");

			}

			//otherwise inform user that it was in position already
			else 
				UserMessages.AddString("Turnout already in position");

			break;

		}



		//check if click was on outsideturnout
		if (abs(point.x - OutsideTurnouts[i].Location.x) < radius &&
			abs(point.y - OutsideTurnouts[i].Location.y) < radius)

		{
			//if it was red try to toggle it
			if (OutsideTurnouts[i].Color == RED)
			{
				if (!MyResourceManager->ToggleTurnout(i))
					UserMessages.AddString("Cannot toggle Train is Blocking");

			}

			//otherwise tell user it was in position already
			else 
				UserMessages.AddString("Turnout already in position");

			break;

		}


	}

}

//////////////////////////////////////////////////////////////////////
//Loads the Init Colors and Postions of Turnouts
/////////////////////////////////////////////////////////////////
void CUserScreen::LoadTurnouts()
{

	OutsideTurnouts[0].Location = UpperLeftBitmap + CPoint(170, 330);
	OutsideTurnouts[0].ColorWhenStraight = GREEN;
	OutsideTurnouts[0].ColorWhenCurved = RED;


	OutsideTurnouts[1].Location = UpperLeftBitmap + CPoint(10, 200);
	OutsideTurnouts[1].ColorWhenStraight = GREEN;
	OutsideTurnouts[1].ColorWhenCurved = RED;


	OutsideTurnouts[2].Location = UpperLeftBitmap + CPoint(470, 10);
	OutsideTurnouts[2].ColorWhenStraight = GREEN;
	OutsideTurnouts[2].ColorWhenCurved = RED;


	OutsideTurnouts[3].Location = UpperLeftBitmap + CPoint(650, 185);
	OutsideTurnouts[3].ColorWhenStraight = GREEN;
	OutsideTurnouts[3].ColorWhenCurved = RED;


	OutsideTurnouts[4].Location = UpperLeftBitmap+ CPoint(485, 360);
	OutsideTurnouts[4].ColorWhenStraight = GREEN;
	OutsideTurnouts[4].ColorWhenCurved = RED;


	OutsideTurnouts[5].Location = UpperLeftBitmap+ CPoint(225, 315);
	OutsideTurnouts[5].ColorWhenStraight = RED;
	OutsideTurnouts[5].ColorWhenCurved = GREEN;


	InsideTurnouts[0].Location = UpperLeftBitmap + CPoint(170, 310);
	InsideTurnouts[0].ColorWhenStraight = RED;
	InsideTurnouts[0].ColorWhenCurved = GREEN;


	InsideTurnouts[1].Location = UpperLeftBitmap+ CPoint(50, 200);
	InsideTurnouts[1].ColorWhenStraight = RED;
	InsideTurnouts[1].ColorWhenCurved = GREEN;


	InsideTurnouts[2].Location = UpperLeftBitmap+ CPoint(470, 45);
	InsideTurnouts[2].ColorWhenStraight = RED;
	InsideTurnouts[2].ColorWhenCurved = GREEN;


	InsideTurnouts[3].Location = UpperLeftBitmap+ CPoint(605, 185);
	InsideTurnouts[3].ColorWhenStraight = RED;
	InsideTurnouts[3].ColorWhenCurved = GREEN;


	InsideTurnouts[4].Location = UpperLeftBitmap+ CPoint(485, 318);
	InsideTurnouts[4].ColorWhenStraight = RED;
	InsideTurnouts[4].ColorWhenCurved = GREEN;


	InsideTurnouts[5].Location = UpperLeftBitmap+ CPoint(215, 280);
	InsideTurnouts[5].ColorWhenStraight = GREEN;
	InsideTurnouts[5].ColorWhenCurved = RED;


}

////////////////////////////////////////////////////////////////////////
//Loads the positions of the CStatic Boxes to show ownership of photocells
///////////////////////////////////////////////////////////////////////////
void CUserScreen::LoadPhotocellTextBoxes()
{

	
	PhotocellOwnership[0].Location = UpperLeftBitmap + CPoint(28, 313);

	PhotocellOwnership[1].Location = UpperLeftBitmap + CPoint(79, 254);

	PhotocellOwnership[2].Location = UpperLeftBitmap + CPoint(10, 74);

	PhotocellOwnership[3].Location = UpperLeftBitmap + CPoint(244, 305);

	PhotocellOwnership[4].Location = UpperLeftBitmap + CPoint(268, 230);

	PhotocellOwnership[5].Location = UpperLeftBitmap + CPoint(268, 85);

	PhotocellOwnership[6].Location = UpperLeftBitmap + CPoint(317, 184);

	PhotocellOwnership[7].Location = UpperLeftBitmap + CPoint(370, 3);

	PhotocellOwnership[8].Location = UpperLeftBitmap + CPoint(378, 351);

	PhotocellOwnership[9].Location = UpperLeftBitmap + CPoint(392, 298);

	PhotocellOwnership[10].Location = UpperLeftBitmap + CPoint(364, 276);

	PhotocellOwnership[11].Location = UpperLeftBitmap + CPoint(392, 208);

	PhotocellOwnership[12].Location = UpperLeftBitmap + CPoint(402, 106);
	 
	PhotocellOwnership[13].Location = UpperLeftBitmap + CPoint(596, 317);

	PhotocellOwnership[14].Location = UpperLeftBitmap + CPoint(557, 242);

	PhotocellOwnership[15].Location = UpperLeftBitmap + CPoint(604, 47);


}


///////////////////////////////////////////////////////////////////////////
//Loads the Positon of the photocells
//////////////////////////////////////////////////////////////////////////
void CUserScreen::LoadPhotocells()
{

	PhotocellPoints[0].Location = UpperLeftBitmap + CPoint(66, 300);
	PhotocellPoints[0].Color = WHITE;

	PhotocellPoints[1].Location = UpperLeftBitmap + CPoint(71, 270);
	PhotocellPoints[1].Color = WHITE;

	PhotocellPoints[2].Location = UpperLeftBitmap + CPoint(47, 96);
	PhotocellPoints[2].Color = WHITE;

	PhotocellPoints[3].Location = UpperLeftBitmap + CPoint(245, 324);
	PhotocellPoints[3].Color = WHITE;

	PhotocellPoints[4].Location = UpperLeftBitmap + CPoint(266, 217);
	PhotocellPoints[4].Color = WHITE;

	PhotocellPoints[5].Location = UpperLeftBitmap + CPoint(263, 103);
	PhotocellPoints[5].Color = WHITE;

	PhotocellPoints[6].Location = UpperLeftBitmap + CPoint(322, 164);
	PhotocellPoints[6].Color = WHITE;

	PhotocellPoints[7].Location = UpperLeftBitmap + CPoint(380, 25);
	PhotocellPoints[7].Color = WHITE;

	PhotocellPoints[8].Location = UpperLeftBitmap + CPoint(389, 342);
	PhotocellPoints[8].Color = WHITE;

	PhotocellPoints[9].Location = UpperLeftBitmap + CPoint(403, 323);
	PhotocellPoints[9].Color = WHITE;

	PhotocellPoints[10].Location = UpperLeftBitmap + CPoint(371, 301);
	PhotocellPoints[10].Color = WHITE;

	PhotocellPoints[11].Location = UpperLeftBitmap + CPoint(383, 234);
	PhotocellPoints[11].Color = WHITE;

	PhotocellPoints[12].Location = UpperLeftBitmap + CPoint(393, 97);
	PhotocellPoints[12].Color = WHITE;

	PhotocellPoints[13].Location = UpperLeftBitmap + CPoint(590, 303);
	PhotocellPoints[13].Color = WHITE;

	PhotocellPoints[14].Location = UpperLeftBitmap + CPoint(576, 265);
	PhotocellPoints[14].Color = WHITE;

	PhotocellPoints[15].Location = UpperLeftBitmap + CPoint(600, 75);
	PhotocellPoints[15].Color = WHITE;

	
}

/////////////////////////////////////////////////////////////
//Finds the larger of two integers
////////////////////////////////////////////////////////////
int FindMax(int x, int y)
{

	if (x > y)
		return x;
	else
		return y;

}
/*****************************************************************************/
/* added by bmcvey 6-27-01 to facilitate adding trains using modeless dialog */
/*****************************************************************************/
/*****************************************************************************
  OnAddTrain - invoked when AddTrain, Using Dialog chosen from menu and
                 popup menu, respectively
			 - sets focus if already exists
			 - allocates, creates, displays and sets focus if did not exist
			 
*****************************************************************************/
void CUserScreen::OnAddTrain() 
{	

	// If the dialog box already exists, display it.
	if (myATDlg != NULL)
		myATDlg->SetFocus ();

	else   // If the dialog box doesn't already exist, create it.
    {
		myATDlg = new CAddTrainDialog ;
	    myATDlg->Create (IDD_NEWEXISTINGTRAIN);
		myATDlg->ShowWindow(SW_SHOW);
		myATDlg->SetFocus ();
	}

}

/*****************************************************************************
  OnAddTrainFromFile - invoked when AddTrain, From File chosen from menu and
                 popup menu, respectively
			     - copied from JV's code 
			     - should add code to validate file entry
				 - should add code to request file name or allow to
				    search directory for it
				 - should allow user to enter train direction and track direction
				    currently forces FORWARD and CLOCKWISE
		(added)  - should REALLY deallocate train object not able to be added to
				   track ... as it is NumTrains not incremented, so ptr overwritten
				   resulting in memory leak
*****************************************************************************/
afx_msg void CUserScreen::OnAddTrainFromFile()
{
		ifstream in;

	in.open("TrainInit.sys"); 
	//read from the TrainInit.sys file to find the starting
	//properties of trains

	if (!in)
	{
		MessageBox("Error opening TrainInit.sys", "Error");
		return;
	}

	//used to get data from the startup file
	int Trainid;
	char TrainName[30];
	int TPhotocellHas;
	int TPhotocellNext;
	int TTOHas;
	int TMode;
	int MaxSpeed;
	int DirectionAroundTrack;
	int Direction;
	char TFileName[80];
 
	//read from file 
	while (!in.eof())
	{
		//the data of the train
		in >> Trainid  >> TrainName >> TPhotocellHas >> TPhotocellNext 
		   >> TTOHas >> TMode >> MaxSpeed >> DirectionAroundTrack >> Direction;

		//DirectionAroundTrack = GetDirectionAroundTrack(TPhotocellHas, TPhotocellNext);
		
		//create an interactive train
		if (TMode == INTERACTIVE)
		{

			AllTrains[NumTrains] = new Train(Trainid,TrainName, TPhotocellHas, 
											 TPhotocellNext, TTOHas, TMode, "" , 
											 MaxSpeed, DirectionAroundTrack, Direction);
			
		}

		else if (TMode == BATCH)
		{
			//create a Batch Train

			in >> TFileName;  //need a file if in batch mode
			AllTrains[NumTrains] = new Train(Trainid,TrainName, TPhotocellHas, 
											 TPhotocellNext, TTOHas, TMode, TFileName, 
											 MaxSpeed, DirectionAroundTrack, Direction);

		}

		// if the add goes well get a user interface for the train
		if (MyResourceManager->AddTrainToTrack(AllTrains[NumTrains]))
		{
			TrainControls[NumTrains] = new CTrainControl(AllTrains[NumTrains], MyResourceManager);
			TrainControls[NumTrains]->ShowWindow(SW_SHOWNORMAL);
			TrainControls[NumTrains]->UpdateWindow();

			NumTrains++;  //up the train counter by 1
			char message[30];  //bmcvey
			char temp[10];
		    strcpy(message, "Train ");
		    strcat(message,  itoa(Trainid, temp, 10));
		    strcat(message, " has been added to the track.");
		    UserMessages.AddString(message);
		}
		else 
		{
			delete AllTrains[NumTrains];  //bmcvey
			AllTrains[NumTrains] = NULL;
			UserMessages.AddString("Unable to add train to track.");
		}

	}
}

/*****************************************************************************
  OnApply - invoked when message sent by CAddTrainDialog is received to
             create a train ..
		  - data to initialize train is in lParam (ptr to TRAINPROP struct)
		  - the rest is copied from JV's code 
			     - CAddTrainDialog validates train info but does
				   not check against already allocated photocells (AddtoTrack does)
		(added)  - should REALLY deallocate train object not able to be added to
				   track ... as it is NumTrains not incremented, so ptr overwritten
				   resulting in memory leak
*****************************************************************************/
LRESULT CUserScreen::OnApply (WPARAM wParam, LPARAM lParam)
{
	TRAINPROP * p = (TRAINPROP*)lParam;
   
   
  /* really just RM->AddTrain(all properties);
      but trying to determine if all data back is correct!
  */
   /*
   char temp[10];
   	itoa(p->TrainID, temp, 10);
	MessageBox("TrainID", temp);

//	strcpy (name, TrainName);
	MessageBox("TrainName", p->TrainName);

	itoa(p->Mode, temp, 10);
	MessageBox("Mode", temp);
	*/
/*	itoa(p->Direction, temp, 10);
	MessageBox("Direction", temp);

	itoa(p->DirAroundTrack, temp, 10);
	MessageBox("DirAroundTrack", temp);
	*/
/*
	itoa(p->MaxSpeed, temp, 10);
	MessageBox("MaxSpeed", temp);

	itoa(p->PCHas, temp, 10);
	MessageBox("PCHas", temp);
	
	itoa(p->NextPC, temp, 10);
	MessageBox("NextPC", temp);

	itoa(p->TOHas, temp, 10);
	MessageBox("TOHas", temp);

	MessageBox("FileName", p->FileName);
	*/
/**************************************************************/
		char temp[10];
		if (p->Mode == INTERACTIVE)
		{

			AllTrains[NumTrains] = new Train(p->TrainID,p->TrainName, p->PCHas, 
				p->NextPC, p->TOHas, p->Mode, "", p->MaxSpeed, p->DirAroundTrack, p->Direction);
			
		}

		else if (p->Mode == BATCH)
		{
			//create a Batch Train

			AllTrains[NumTrains] = new Train(p->TrainID,p->TrainName, p->PCHas, 
				p->NextPC, p->TOHas, p->Mode, p->FileName, p->MaxSpeed, p->DirAroundTrack, p->Direction);

		}

		// if the add goes well get a user interface for the train
		if (MyResourceManager->AddTrainToTrack(AllTrains[NumTrains]))
		{
			TrainControls[NumTrains] = new CTrainControl(AllTrains[NumTrains], MyResourceManager);
			TrainControls[NumTrains]->ShowWindow(SW_SHOWNORMAL);
			TrainControls[NumTrains]->UpdateWindow();
  
			NumTrains++;  //up the train counter by 1
			char message[30];
		    strcpy(message, "Train ");
		    strcat(message,  itoa(p->TrainID, temp, 10));
		    strcat(message, " has been added to the track.");
		    UserMessages.AddString(message);
			Invalidate(); // force redraw of screen
		}
		else 
		{
			delete AllTrains[NumTrains];   //bmcvey
			UserMessages.AddString("Unable to add train to track.");
			AllTrains[NumTrains] = NULL;
		}

	//BringWindowToTop();  //didnt work to bring userscreen in front of addtraindialog
	//	SetForegroundWindow(); //ditto

    return 0;
}

LRESULT CUserScreen::OnDialogDestroyed (WPARAM wParam, LPARAM lParam)
{
   //MessageBox ("OndialogDestroyed", "Childview");
	myATDlg = NULL;
	return 0;
}

LRESULT CUserScreen::OnTrainDestroyed (WPARAM wParam, LPARAM lParam)
{
  
	  char temp[10];
	  int id = *((int *)lParam);
	 // MessageBox("id of train", itoa( id, temp, 10));
	  // find id of train in AllTrains and make ptr NULL and TrainControl
	  // ptr NULL, and shift up
	  // all remaining trains and their controls, decrease NumTrains
	  // call RM->DeleteTrainFromTrack to update resources and actually
	  // delete the memory allocated to the train itself

	  int i = 0;
      bool found = false;
	  while (i < NumTrains && !found)
	  {
		  if (AllTrains[i]->GetId() == id)
			  found = true;
		  else
			  i++;
	  }
	  if (found)
	  {
		  char message[30];
		  strcpy(message, "Train ");
		  strcat(message,  itoa(id, temp, 10));
		  strcat(message, " has been removed from track.");
		  UserMessages.AddString(message);
	      int j;
	      for (j = i; j < NumTrains-1; j++)    // watch out for NumTrains == MAXTRAINS
		  {
		      AllTrains[j] = AllTrains[j+1];
		      TrainControls[j] = TrainControls[j+1];
		  }
		  AllTrains[NumTrains-1] = NULL;
		  TrainControls[NumTrains-1] = NULL;
		  NumTrains--;
		 MessageBox("ontraindestroyed", "both array entries NULL");
	      MyResourceManager->DeleteTrainFromTrack(id);
		  Invalidate(); // force redraw of screen (track)
		  
	  }

	return 0;
}

/*********** Laura - 4/6/02 *******************
int GetDirectionAroundTrack(int PCHas, int PCNext){
	int dir;
	if(CC[PCHas].FindPCNext(0) == PCNext)
		dir = 0;
	else if(CC[PCHas].FindPCNext(1) == PCNext)
		dir = 0;
	else if (CW[PCHas].FindPCNext(0) == PCNext)
		dir = 1;
	else if (CW[PCHas].FindPCNext(1) == PCNext)
		dir = 1;
	return dir;
}
************************************************/

/*****************************************************************************/

//////////////////////////////////////////////////////////////
//Sets message map to call functions when events are received
////////////////////////////////////////////////////////////////
BEGIN_MESSAGE_MAP(CUserScreen, CFrameWnd)

	ON_WM_DESTROY()  //bmcvey -- on destroy was never previously invoked
	
	ON_WM_CREATE()

	ON_WM_TIMER()    

	ON_WM_PAINT()   

	ON_COMMAND(IDC_ENABLETRACKBUTTON, OnToggleEnabledCourse)

	ON_COMMAND(IDM_EXIT, OnExit)

	ON_WM_LBUTTONDOWN ()

/*****************************************************************************/
/* added by bmcvey 6-27-01 to facilitate adding trains using modeless dialog */
	ON_COMMAND(IDC_ADD_TRAIN, OnAddTrain)
	ON_COMMAND(IDC_ADD_FROM_FILE, OnAddTrainFromFile)
	ON_MESSAGE (WM_USER_APPLY, OnApply)
	ON_MESSAGE (WM_USER_DIALOG_DESTROYED, OnDialogDestroyed)
	ON_MESSAGE (WM_USER_TRAINCONTROL_DESTROYED, OnTrainDestroyed) // bmcvey
/*****************************************************************************/
END_MESSAGE_MAP()