#include <iomanip>
#include "TrackList.h"
#include "ResourceManager.h"
#include "ccr.h"
#include "train.h"
#include "MiscFcns.h"
using namespace std;

//******************************************************************
// AUTHOR: Laura Weiland
// DATE: Spring 2002
//
// Below are the functions necessary to detect a deadlocked system 
// state.
//******************************************************************


int ResourceManager::Check4Deadlock(int trainID, int deadids[], int &deadcount){
//******************************************************************
// PURPOSE: The purpose of this function is to check for deadlock
//          in the list of blocked trains.
//
// INPUT: The train id of the train that has just been added.
//
// OUTPUT: * 0 if deadlock was NOT detected
//		   * 1 if deadlock was detected
//		   * An array of train ids participating in deadlock is 
//			 returned in decending order  (deadids[])
//		   * A count of the number of trains participating in 
//			 deadlock is returned  (deadcount)
//
// DATE: Spring 2002
//******************************************************************
	
	int deadlock = 0;
	int table[16][2];
	int count = 0;
	int start, position;

	Output << "\nCHECKING FOR DEADLOCK...\n";
	position = CreateTable(table, trainID);
	Output << "\nArray of Blocked trains - \n";
	
	// Output array of blocked train info to the log file
	Output << setw(10) << "Photocell" << setw(12) << "Train Id" << setw(12) 
		   << "Blocking" << endl << setw(7) << "Has" << setw(28) << "Photocell";
	for(int i=0; i<16; i++){
		Output << "\n" << setw(6) << i << setw(3) << " ";
		for(int j=0; j<2; j++){
			Output << setw(10) << table[i][j];
		}
	}

	//********************************************************************
	// initialize the array of train ids participating in deadlock with a  
	// number greater than used id numbers - since the short id numbers  
	// are used for each train, 1000 will never be used
	//********************************************************************
	for(i=0; i<16; i++)
		deadids[i] = 1000;

	start = position;

	//********************************************************************
	// Cycle through the indexed array looking for a circular pattern.
	// Fill the array of train ids that are participating in deadlock.
	//********************************************************************
	while(count < 16 && position != -1 && start != table[position][1]){
		deadids[count] = table[position][0];
		position = table[position][1];
		count++;
	}
	
	//********************************************************************
	// If the last position made the circle complete, add the train number
	// of the train at that postion to the list of train ids as well
	//********************************************************************
	if(position != -1){
		deadids[count] = table[position][0];
		count++;
	}

	sortarray(deadids, count);
	
	deadcount = count;
	
	// if a complete circle has been detected, deadlock has been detected
	if(position != -1 && start == table[position][1])
		deadlock = 1;

	return deadlock;
}



int ResourceManager::CreateTable(int table[][2], int trainID){
//******************************************************************
// PURPOSE: The purpose of this function is to create an indexed 
//          array (by nextPC - the PC the train is currently covering) 
//			of train ids and their requesting/blocked photocells from 
//			the list of blocked trains.
//
// INPUT: The train id of the train that has just been added and the
//		  table that needs filling.
//
// OUTPUT: position - the position in the array of the offending  
//					  blocked train.
//
// DATE: Spring 2002
//******************************************************************

	int position;
	Train *p;

	//Initialize Table
	for(int i=0; i<16; i++){
		for(int j=0; j<2; j++){
			table[i][j] = -1;
		}
	}
	
	//**************************************************************
	// TABLE:
	//                 -------------------------------
	//   PhotocellNext | TrainID | BlockingPhotocell |
	//      (Index)    -------------------------------
	//         0       |   16    |         1         |
	//                 -------------------------------
	//**************************************************************

	p = BlockedTrains.top -> link;
		
	while(p != BlockedTrains.top){

		//**********************************************************
		// p circles through the linked list once, updating the 
		// table. 
		//**********************************************************

		table[p->GetPhotocellNext()][0] = p -> GetId(); 
		table[p->GetPhotocellNext()][1] = p -> GetBlockingPhotocell(); 
		
		//**********************************************************
		// checks to see if this is the train that was just put into
		// the blocked trains list
		//**********************************************************
		if(trainID == p->GetId())
			position = p->GetPhotocellNext();

		p = p->link;

	}

	return position;
}

void sortarray(int deadids[], int deadcount){
//******************************************************************
// PURPOSE: The purpose of this function is to sort the array of  
//          train ids participating in deadlock into an decending 
//			numberic order.
//
// INPUT: An array of train ids participating in deadlock and the
//		  number of trains participating.
//
// OUTPUT: a sorted array  
//
// DATE: Spring 2002
//******************************************************************

	int largest, first, i;
	for(first = 0; first < deadcount-1; first++){
		largest = first;
		for(i = first + 1; i < deadcount; i++){
			if(deadids[i] > deadids[largest])
				largest = i;
		}
		swap(deadids[first],deadids[largest]);
	}
	return;
}
