#include <fstream>
#include <iomanip>  //BM 2-14-02 added to format ResourceLogFile
#include "TrackList.h"
#include "ResourceManager.h"
#include "MiscFcns.h"
#include "ccr.h"
#include "DCC_comm.h"
#include <string>
using namespace std;

/////////////////////////////////////////////////////////////////////////////
//Constructs a resourcemanager.  This is called when the user screeen is started
//and it creates a pointer to resoucemanager.  This will create a window that starts
//invisible, but fires timers to check if photocells are covered.  It initializes the
//ports for the turnouts and photocells useing functions defined in ccr.h and 
//coded in ccr.dll which is pointed to by the dp.lib.  Init the data structures
//of PreviousPCState and CurrentPCState arrays to be 0 to indicate that no photocells 
//have been triggered.  Also PhotocellManager array is init to 0 to represent that 
//no photocells bounding sections of track have been allocated.  Loads the track
//definition with a call to LoadTrack() and Sets a timer to fire every half second
////////////////////////////////////////////////////////////////////////////////	
		
ResourceManager::ResourceManager()
{

	CRect Rect(0,0,500, 500);  //define oringal window
	Create(0, "ResourceManager", WS_OVERLAPPEDWINDOW, Rect, NULL, 0);  
	//create new window

 	CRect MyRect(0, 0, m_cxChar*40,  m_cyChar * 10); 
	// comes back with m_cxChar and my_cyChar defined set up new rect for new window
										

	CalcWindowRect(&MyRect);  // calc pixel width and height for window

	SetWindowPos(NULL, 0, 0, MyRect.Width() , MyRect.Height(), 
		         SWP_NOZORDER | SWP_NOMOVE | SWP_NOREDRAW);

	
	int PortStatus = Open_Ports(); 
	//open ports to communicate with photocells and turnouts

	int InitStatus = Initialize_Board(1, 0, 0, 0, 0);
	InitStatus = Initialize_Board(2, 1, 1, 1, 1);
	InitStatus = Initialize_Board(3, 1, 1, 1, 1);
    InitStatus = Initialize_Board(4, 1, 1, 1, 1);
	//Init the four green boards with the photocells and turnouts
	//starts the turnouts in correct default positions.	
	
	MyDCC_open("COM1");
	MyDCC_reset();

	

	
	for (int i = 0; i<MAXPHOTOCELLS; i++)
	{
		PreviousPCState[i] = 0;
		//init previous state to indicate no photocells had been tripped in past
		CurrentPCState[i] = 0;
		//init current state of photocells to indicated none have been tripped
		//right now
		PhotocellManager[i] = 0;
		//Set photocell manager to indicated that no photocells bounding a 
		//secion of track have been allocated to any train.
		//This array is 0 if no train owns it and the trains id if the train
		//does own it
	}

	for (i = 0; i< MAXTURNOUTS; i++)
		TurnoutMng[i] = 0;
	//Init the turnoutmanager array to be 0 to indicate that no trains own any turnout

	//*************************************************************** 
	// PURPOSE: Init the yellow light array for each train/photocell
	// DATE: Spring 2002 - Deadlock Avoidance
	//***************************************************************
	for(i = 0; i < MAXTRAINS; i++){
		for(int j = 0; j < MAXPHOTOCELLS; j++)
			YellowLights[i][j] = -1;
	}
	//***************************************************************
	
	for(i = 0; i < MAXPHOTOCELLS; i++)
		NumAtYellow[i] = 0;
	//Init the number of trains requesting 

	LoadTrack();  

	TrackEnabled = true;
	// loads the tracklist definitions that provide a function to determine 
	//which photocells and turnouts are related to each other on a track
					
	SetTimer( 1, 500, NULL ); 
	//set a timer to fire every half second to check if photocells have been covered
	//so that the OS can allocated and deallocated resources

	Output.open("ResourceLog.txt");


}

////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////
//Destructor of Resouce Manager--Executed when a resource manager is detroyed
//close all the files we opened and kill the track to stop the trains
////////////////////////////////////////////////////////////////////////////
ResourceManager::~ResourceManager()
{


	MyDCC_kill(); 
	//kill the track -- if we don't have a resouce manager we don't have
	//an operating system and the trains could crash

   //Close_Ports() defined in ccr.h and implemented in ccr.dll 
   // which is pointed to be dp.lib
   Close_Ports();  // close the ports to the photocells and turnouts
   Output.close(); //close the ResourceManager log file

   MyDCC_close(); //Close comm port to communicate train commands

 

}


//////////////////////////////////////////////////////////////////////////
//Checks to see if the photocell and turnout that a train is requesting is allocated
//to another train already.  If both the Turnout and photocell are currently 
//not allocated then AllcoateResources is called to allocate the photocell and 
//turnout to the requesting train.  If the resources are not available FALSE 0 is
//returned.  This function will have racing conditions if timers can queue
////////////////////////////////////////////////////////////////////////////////////
int ResourceManager::UpdateResources(Train * T, int pc, int to)
{

	if (PhotocellManager[pc] == 0)  
	//check if photocellmanager is allocated to a train
	{
		if (to != -1)//if turnout exists (not -1) 
		{
			if (TurnoutMng[to] == 0)//then check if it is allocate to a trian
			{
				AllocateResources(T,pc,to);//if not Allocate the resources
				if (T->GetMode() == BATCH)
					BatchToSwitch(T,pc, to);
				return TRUE; // and return True that the resources were allocated
			}
			else
				return FALSE; //otherwise return false
			
		}
		else //if there are no turnouts to worry about just allocate and return true
		{
			AllocateResources(T,pc,to);
			return TRUE;
		}

	}

	else 
		//if photocell is already allocated to a train can't allocated
		// to this train so return false
		return FALSE;


}

/////////////////////////////////////////////////////////////////////
//This function find the direction of a turnout given the next photocell
//to reach.  Called from UpdateResources and used to find the direction
//a turnout needs to be in using batch mode.  Since this is only called
//when we can update the resources, when we find the correct position
//of the turnouts we will toggle it
//////////////////////////////////////////////////////////////////////
void ResourceManager::BatchToSwitch(Train * T, int pc, int to)
{

	int switchdirection;

	if (T->GetDirectionAroundTrack() == CLOCKWISE)
  		switchdirection = CW[T->PhotocellHas].FindSwitchDirection(pc);
	else		
		switchdirection = CC[T->PhotocellHas].FindSwitchDirection(pc);

	if (switchdirection != Get_TO_Status(to))
		TO_Toggle(to);
}


//////////////////////////////////////////////////////////////////////////////////
//This function is called from UpdateResources() if that function found the
//the train requesting the resources of a photocell and turnout could indeed
//be allocated to the requesting train. This function updates the Oldphotocellhas
//and OldTurnoutHas so that when it comes time to deallocate this trains old
//resources we remember which ones in the array to put 0 values in.  The function
//then updates the Photocell that the train has to the one that it used to want
//next and updates the Photocellnext to be the one that the functions defined
//by tracklist to be the photocell it wanted next.  The function then fills the
//photocellmanager with the Trains Id and fills the turnoutmanager if the turnout
//is exists (ie not -1) with the trains id
//
//Added: Spring 2002
//		 This function now also manages the yellow lights list for deadlock 
//		 avoidance.
//////////////////////////////////////////////////////////////////////////////////

void ResourceManager::AllocateResources(Train * T, int pc, int to)
{


		T->OldPhotocellHas = T->PhotocellHas; 
		//update the oldphotocellhas to remember which one to deallocate when
		//the train uncovers the photocell

		T->OldTurnoutHas = T->TurnoutHas;
		// update the oldturnouthas to remember which one to deallocate when 
		//the train uncovers the phtocell

		T->PhotocellHas = T->PhotocellNext; 
		//update pchas with next since we've arrived

		T->PhotocellNext = pc; 
		//update the next photocel with the one that was found
		//by the FindNextFunction in TrackList

		T->TurnoutHas = to; // and update the turnout that the Train now owns

		PhotocellManager[T->PhotocellHas] = T->Id;  
		//Fill the array in the photocellhas location with the Trains id
		PhotocellManager[T->PhotocellNext] = T->Id;
		//Fill the array in the photocellnext location with the Trains id


		if (to != -1){
			TurnoutMng[to] = T->Id;
			//if a turnout exists fill the array at the location with the Trains id
		}
		

		int pc1, pc2, pc3, pc4, to1, removed;

		if(T->GetDirectionAroundTrack() == CLOCKWISE){
			//one PC ahead in current direction of next TO
			pc1 = CW[T->PhotocellNext].FindPCNext();

			//one TO ahead
			to1 = CW[T->PhotocellNext].FindToNext();
			
			//one PC ahead in opposite direction of next TO
			if(to1 != -1)
				pc2 = CW[T->PhotocellNext].FindPCNext(TurnoutOpposite(to1));
			else
				pc2 = pc1;
			
			//PC following current TO
			pc3 = CW[T->PhotocellHas].FindPCNext(); 

			//PC following current TO
			if(T->TurnoutHas != -1)
				pc4 = CW[T->PhotocellHas].FindPCNext(TurnoutOpposite(T->TurnoutHas));
			else
				pc4 = -1;
		}
		else{
			//one PC ahead in current direction of next TO
			pc1 = CC[T->PhotocellNext].FindPCNext(); 

			//one TO ahead
			to1 = CC[T->PhotocellNext].FindToNext();
			
			//one PC ahead in opposite direction of next TO
			if(to1 != -1)
				pc2 = CC[T->PhotocellNext].FindPCNext(TurnoutOpposite(to1));
			else
				pc2 = pc1;
			
			//PC following current TO
			pc3 = CC[T->PhotocellHas].FindPCNext();
			
			//PC following current TO
			if(T->TurnoutHas != -1)
				pc4 = CC[T->PhotocellHas].FindPCNext(TurnoutOpposite(T->TurnoutHas));
			else
				pc4 = -1;
		}
		
		// Remove the Yellow light from the last allocation
		removed = 0;
		if(pc3 != -1)
			removed = RemoveFromYellow(pc3, T->Id);

		//**********************************************************************
		// If a train was blocked due to a cautious yellow light (i.e. a PC that 
		// was marked with a yellow light that the train never actually traveled
		// to) and can now be returned to the active train list, call 
		// ReleaseBlockedTrains.  
		//
		// NOTE: A -2 was used as the turnout number because there is no turnout 
		// -2, thus a train will only be moved to active if it was blocked by 
		// the yellow light on this PC
		//**********************************************************************
		if(removed && pc3 != pc){
			ReleaseBlockedTrains(pc3, -2);
			Output << "\nTrain Id: " << T->Id 
				   << " releasing blocked trains at photocell " << pc3 << ".";
		}
		
		// Remove the Yellow light from the last allocation
		removed = 0;
		if(pc4 != -1)
			removed = RemoveFromYellow(pc4, T->Id);

		//**********************************************************************
		// If a train was blocked due to a cautious yellow light (i.e. a PC that 
		// was marked with a yellow light that the train never actually traveled
		// to) and can now be returned to the active train list, call 
		// ReleaseBlockedTrains.  
		//
		// NOTE: A -2 was used as the turnout number because there is no turnout 
		// -2, thus a train will only be moved to active if it was blocked by 
		// the yellow light on this PC
		//**********************************************************************
		if(removed && pc4 != pc){
			ReleaseBlockedTrains(pc4, -2);
			Output << "\nTrain Id: " << T->Id 
				   << " releasing blocked trains at photocell " << pc4 << ".";
		}

		
		// Mark the next sections of track with yellow lights
		if(pc1 != -1)
			AddToYellow(pc1, T->Id);
		if(pc2 != pc1 && pc2 != -1)
			AddToYellow(pc2, T->Id);

		//Output to our log file
		Output << "\n\nTrain " << T->Id << " was allocated photocells: " 
			   << T->PhotocellHas << " and " << T->PhotocellNext 
			   << " and turnouts: " << T->TurnoutHas << endl;
		
		//print the trains information
		TrainPrint(T);
		//print the photocell and turnout manager to resource log
		ResourceTablesPrint();
		//print the array of yellow lights allocated
		DisplayYellow();

}


//////////////////////////////////////////////////////////////////////////////
//This function is called from HandleDeparture and set the PhotocellManager 
//and TurnoutMng back to 0 in the position defined by a trains OldphotocellHas
//Old TurnoutHas which are the resources that need to be deallocated when a 
//photocell becomes uncovered
//
//Added: Spring 2002
//		 PC and TO are only deallocated if the train that is 
//		 requesting the deallocation actually owns the PC or TO
//////////////////////////////////////////////////////////////////////////////
void ResourceManager::DeallocateResources(Train * T)
{
	int deallocated = 0;
	/********** Laura - Spring 2002 ***********/
  		if(PhotocellManager[T->PhotocellHas] == T->GetId()){
			PhotocellManager[T->PhotocellHas] = 0;
			deallocated = 1;
		}
	   //set the Photocellmanager in position oldphotocellhas to 0
		if (T->TurnoutHas != -1)
		{	/********** Laura - Spring 2002 ***********/
			if(TurnoutMng[T->TurnoutHas] == T->GetId()){
				TurnoutMng[T->TurnoutHas] = 0;
				deallocated = 1;
			}
			//set the TurnoutMng in position OldTurnoutHas to 0
		}

		if(deallocated)
			ReleaseBlockedTrains(T->PhotocellHas, T->TurnoutHas);
		//see if the photocell just deallocated can wake up any blocked train
		//and move them to the active list again.


		// output to the resource log
		Output << "\nTrain " << T->Id << " deallocated photocell: " 
			   << T->PhotocellHas << " and turnout: " << T->TurnoutHas << endl;

		TrainPrint(T);
		ResourceTablesPrint();
		DisplayYellow();
}

////////////////////////////////////////////////////////////////////////
//This function prints the contents of the PhotocellManger and TurnoutMng
//arrays to the resourceManager log file
/////////////////////////////////////////////////////////////////////////
void ResourceManager::ResourceTablesPrint()
{
// BM - 2-14-02 - changed format of information displayed and
//				  added to output the contents of CurPC and PrevPC
	int i;
	Output << "Photocells      : ";
	for (i = 0; i<MAXPHOTOCELLS; i++) 
		Output << setw(3) << i;
	Output << endl;

	Output << "CurrentPCState  : ";
	for (i = 0; i<MAXPHOTOCELLS; i++) 
		Output << setw(3) << CurrentPCState[i];
	Output << endl;

	Output << "PreviousPCState : ";
	for (i = 0; i<MAXPHOTOCELLS; i++) 
		Output << setw(3) << PreviousPCState[i];
	Output << endl;

	Output << "PCManager       : ";
	for (i = 0; i<MAXPHOTOCELLS; i++) 
		Output << setw(3) << PhotocellManager[i];
	Output << endl;

	Output << "TurnoutMgr      : ";
	for (i = 0; i<MAXTURNOUTS; i++)
		Output << setw(3) << TurnoutMng[i];
	Output << endl;

}



///////////////////////////////////////////////////////////////////////////////////
//This function is called from the UserScreen when the user wishes to add a train
//to the track.  Checks to make sure that the trains id greater than 0 and unique
//in the activetrainlist. If Train id is bad function returns -2. Then checks if 
//the train can be added at present time by checking if the trains defined starting
//location is allocated to another train. If the train cannot be added right now 
//function returns a -1. If the train can be added to track it is added to the 
//activetrainlist and the photocellmanager array are updated to have the new Train 
//allocated to its defined starting postion. If all goes well function returns 1
//
//Added: Spring 2002
//		 When train is initialized, update the yellow lights array to reflect
//		 the added train.
///////////////////////////////////////////////////////////////////////////////////
int ResourceManager::AddTrainToTrack(Train * T)
{
 
	if (T->GetId() > 0 &&  ActiveTrains.Find(T->GetId()) == NULL)
		//check to see if the id is positive and unique
	{
		
		if (PhotocellManager[T->PhotocellHas  /*DEFPHOTOCELLHAS*/] == 0 && 
			PhotocellManager[T->PhotocellNext/*DEFPHOTOCELLNEXT*/] == 0)
		{ // checks if train can be added by checking if the resources it needs to 
		  //start are not already allocated to another train

			ActiveTrains.Add(T); // add train to activelist
			T->BlockedStatus = ACTIVE;
			
			PhotocellManager[T->PhotocellHas/*DEFPHOTOCELLHAS*/] = T->GetId();
			PhotocellManager[T->PhotocellNext/*DEFPHOTOCELLNEXT*/] = T->GetId();
			//fill the photocell manager with trains id to signal that 
			//the train now owns these resources(two photocells that bound
			//a section of track

			if (T->TurnoutHas != -1)
				TurnoutMng[T->TurnoutHas] = T->GetId();

			int pc1, pc2, to1;
			
			// Get the PC's on the next section of track and update with yellow flags
			if(T->GetDirectionAroundTrack() == CLOCKWISE){
				//one PC ahead in current direction of next TO
				pc1 = CW[T->PhotocellNext].FindPCNext();

				//one TO ahead
				to1 = CW[T->PhotocellNext].FindToNext();
				
				//one PC ahead in opposite direction of next TO
				if(to1 != -1)
					pc2 = CW[T->PhotocellNext].FindPCNext(TurnoutOpposite(to1));
				else
					pc2 = pc1;
			}
			else{
				//one PC ahead in current direction of next TO
				pc1 = CC[T->PhotocellNext].FindPCNext();

				//one TO ahead
				to1 = CC[T->PhotocellNext].FindToNext();

				//one PC ahead in opposite direction of next TO
				if(to1 != -1)
					pc2 = CC[T->PhotocellNext].FindPCNext(TurnoutOpposite(to1));
				else
					pc2 = pc1;
			}

			//Output to resource log
			Output << "Train " << T->Id << " was added to the track given photocells: "
				   << T->PhotocellHas << " and " << T->PhotocellNext << " and turnouts: "
				   << T->TurnoutHas << endl;

			//Update yellow flags for new train
			if(pc1 != -1)
				AddToYellow(pc1, T->Id);
			if(pc2 != pc1 && pc2 != -1)
				AddToYellow(pc2, T->Id);

			//if the train was added to an UNSAFE condition, output to the log file
			if(NumAtYellow[T->PhotocellNext] > 0){
				Output << "\n\nTrain " << T->Id 
					   << " was added to an UNSAFE section of track.\n";
			}
			
			// BM to get train to print
			TrainPrint(T);
			ResourceTablesPrint();
			DisplayYellow();
			return 1;  //everything went good
		}

		else
		{
			Output << "Train " << T->Id 
			       << " couldn't be added to the track resources not available" << endl;
			return 0;
		}
		//another train already owns the resources this train would need
		//to start at its defined starting location
	}

	else
	{
		Output << "Train " << T->Id << " Couldn't be added to track because of bad id"
		        << " either negative or not unique" << endl;
		return 0;
	}
	// id is bad either negative or not unique
} 



/////////////////////////////////////////////////////////////////////////////
//PURPOSE: Deletes a train from the track and frees that trains resources
//Looks in both the activetrainlist and blocked list for the id of the train
//to delete and then searches the correct list. Find the train and adjusts
//the photocellmanager and turnout manager to 0 in the resources that the train
//owned prior to being deleted
//
//OUTPUT: return 1 if successful
//		  returns 0 if not successful 
//
//Added: Spring 2002
//		 Deallocate all yellow flags from this train and release any trains
//		 that were blocked by them.
///////////////////////////////////////////////////////////////////////////
int ResourceManager::DeleteTrainFromTrack(int id) 
{

	Train *prev, *p, *MyTop;  //used to keep track of location in lists


	if (ActiveTrains.Find(id) != NULL)
	{

		//if train is found in activeTrainlist 
		MyTop = ActiveTrains.top; //set Mytop to activetrain's top for loop
		prev=ActiveTrains.top; //set prev as activetrainlist's header
		p=ActiveTrains.top-> link; // and p to be first real train in activetrainlist

	}

	else if (BlockedTrains.Find(id) != NULL)
	{

		//if train is found in BlockedTrainlist 
		MyTop = BlockedTrains.top;//set Mytop to blockedtrain's top for loop
		prev = BlockedTrains.top;//set prev as blockedlist's header
		p = BlockedTrains.top;// and p to be first real train in blockedtrainlist

	}

	while (p!=MyTop) 
	//circular header list ends when last node points back to top of list
	{

		if (p->GetId() == id)
		{
		    prev-> link =p -> link;
			//when we find it adjust the one before this trains link to point
			//to the one after it thus taking this train out of the list

			PhotocellManager[p->GetPhotocellHas()] = 0;
			PhotocellManager[p->GetPhotocellNext()] = 0;
			// and fill the photocells that it owned with 0 thus
			//deallocating the trains track allocation

			if (p->GetTurnoutHas() != -1)
				TurnoutMng[p->GetTurnoutHas()] = 0;  
			    // if it owned a turnout deallocate it as well

			int pc1, pc2, pc3, pc4, to1, removed;
			
			// Get all yellow flags associated with this train
			if(p->GetDirectionAroundTrack() == CLOCKWISE){
				//one PC ahead in current direction of next TO
				pc1 = CW[p->PhotocellNext].FindPCNext();

				//one TO ahead
				to1 = CW[p->PhotocellNext].FindToNext(); 
				
				//one PC ahead in opposite direction of TO
				if(to1 != -1)
					pc2 = CW[p->PhotocellNext].FindPCNext(TurnoutOpposite(to1));
				else
					pc2 = pc1;

				//PC following current TO
				pc3 = CW[p->PhotocellHas].FindPCNext(); 
				
				//PC following current TO
				if(p->TurnoutHas != -1)
					pc4 = CW[p->PhotocellHas].FindPCNext(TurnoutOpposite(p->TurnoutHas));
				else
					pc4 = -1;
			}
			else{
				//one PC ahead in current direction of next TO
				pc1 = CC[p->PhotocellNext].FindPCNext();
				
				//one TO ahead
				to1 = CC[p->PhotocellNext].FindToNext();
				if(to1 != -1)
					pc2 = CC[p->PhotocellNext].FindPCNext(TurnoutOpposite(to1));
				else
					pc2 = pc1;

				//PC following current TO
				pc3 = CC[p->PhotocellHas].FindPCNext(); 
				
				//PC following current TO
				if(p->TurnoutHas != -1)
					pc4 = CC[p->PhotocellHas].FindPCNext(TurnoutOpposite(p->TurnoutHas));
				else
					pc4 = -1;
			}
			
			//*****************************************************************
			// For each Yellow light removed, release any blocked trains
			removed = 0;
			if(pc1 != -1)
				removed = RemoveFromYellow(pc1, p->Id);
			if(removed)
				ReleaseBlockedTrains(pc1, to1);
			
			removed = 0;
			if(pc2 != pc1 && pc2 != -1)
				removed = RemoveFromYellow(pc2, p->Id);
			if(removed)
				ReleaseBlockedTrains(pc2, to1);
			
			removed = 0;
			if(pc3 != -1)
				removed = RemoveFromYellow(pc3, p->Id);
			if(removed)
				ReleaseBlockedTrains(pc3, p->TurnoutHas);

			removed = 0;
			if(pc4 != -1)
				removed = RemoveFromYellow(pc4, p->Id);			
			if(removed)
				ReleaseBlockedTrains(pc4, p->TurnoutHas);
			//*****************************************************************
			
			Output << "Train " << p->Id << " was removed from the track and freed"
				   << " photocells: " << p->PhotocellNext << " and " <<p->PhotocellHas
				   << " and turnouts: " << p->TurnoutHas << endl;

		    delete p; // delete the trains node
			return 1; // return 1 for successful
		}

		// have not found it yet continue in list
		prev=p; 
		p=p->link;
	}

	return 0;// not successful return 0
} 



/////////////////////////////////////////////////////////////////////////////////
//This function Takes a Train and moves it from the active train list to the blocked
//train list.  It is called when a train cannot be allocated the resources that 
//it needs to continue and the train needs to be blocked  Called from HandleArrival()
//
//Added: Spring 2002
//	* Yellow lights are deallocated due to the blocked status.
//	* State of system is checked for deadlock - see deadlock fcns in deadlock.cpp
//	* Recovery algorithm put into effect if deadlock was detected
//		- Find the train with the highest id number that can successfully back up
//		  one instruction its sequence.  Change the train's direction and execute 
//		  an arrival in the new direction with the instruction to go back to the 
//		  previous photocell.
//////////////////////////////////////////////////////////////////////////////////

void ResourceManager::ActiveToBlocked(Train * T)
{
	int deadlock = 0;

	Train *prev, *p; 
	//Pointers to trains that keeps track of location is activetrainlist

	prev=ActiveTrains.top;  // prev points to the header node
	p=ActiveTrains.top-> link; // p pointes to the first real train

	while (p!=ActiveTrains.top) 
	//circular header list stops when the last node points to the top
	{
		if (p->GetId() == T->GetId())  //we found our train in the activelist
		{
		    prev-> link =p -> link;  
			// adjust the node before it to point to the one after it so
			// it skips this node thus taking this node out of the activetrainlist
		   
			Train * q, *qprev;  
			//define new pointers to keep track of BlockedTrainList position

			qprev = BlockedTrains.top; //set qprev to header
			q = BlockedTrains.top->link;  // and q to first real train

			while(p->GetId() > q->GetId())
			// loop until we put the id in by numerical order(ascending order)
			{
				qprev = q; 
				q = q->link;
				// didn't find it yet continue on in the blockedtrainlist
				
			}
			
			p->link = q;  
			qprev->link = p; 
			p->BlockedStatus = BLOCKED;
			Output << "Train " << p->Id << " was added to the blocked list " << endl;

			int pc1, pc2, pc3, pc4, to1;

			//deallocate all yellow lights allocated to this train
			if(p->GetDirectionAroundTrack() == CLOCKWISE){
				//one PC ahead in current direction of next TO
				pc1 = CW[p->PhotocellNext].FindPCNext(); 

				//one TO ahead
				to1 = CW[p->PhotocellNext].FindToNext(); 

				//one PC ahead in opposite direction of next TO
				if(to1 != -1)
					pc2 = CW[p->PhotocellNext].FindPCNext(TurnoutOpposite(to1)); 
				else
					pc2 = pc1;

				//PC following current TO
				pc3 = CW[p->PhotocellHas].FindPCNext();

				//PC following current TO
				if(p->TurnoutHas != -1)
					pc4 = CW[p->PhotocellHas].FindPCNext(TurnoutOpposite(p->TurnoutHas)); 
				else
					pc4 = -1;
			}
			else{
				//one PC ahead in current direction of next TO
				pc1 = CC[p->PhotocellNext].FindPCNext();

				//one TO ahead
				to1 = CC[p->PhotocellNext].FindToNext();

				//one PC ahead in opposite direction of next TO
				if(to1 != -1)
					pc2 = CC[p->PhotocellNext].FindPCNext(TurnoutOpposite(to1)); 
				else
					pc2 = pc1;

				//PC following current TO
				pc3 = CC[p->PhotocellHas].FindPCNext(); //pc in opposite branch of last to
				
				//PC following current TO
				if(T->TurnoutHas != -1)
					pc4 = CC[p->PhotocellHas].FindPCNext(TurnoutOpposite(p->TurnoutHas)); 
				else
					pc4 = -1;
			}
		
			// Remove all yellow lights associated with this train
			if(pc1 != -1)
				RemoveFromYellow(pc1, p->Id);
			
			if(pc2 != pc1 && pc2 != -1)
				RemoveFromYellow(pc2, p->Id);
			
			if(pc3 != -1)
				RemoveFromYellow(pc3, p->Id);
			
			if(pc4 != -1)
				RemoveFromYellow(pc4, p->Id);
			
			
			// Move back one instruction in the instruction sequence
			if(p->GetMode() == BATCH){
				if(p->cursor == 0)
					p->cursor = p->LinesRead;
				else
					p->cursor --;
			}
						
			int deadids[16];
			int deadcount = 0;

			// Checking for deadlock
			deadlock = Check4Deadlock(p->Id, deadids, deadcount);

			if(deadlock){
				//***************************************
				// Deadlock was detected...start recovery
				//***************************************

				Output << "\n\nDEADLOCK was detected!\n\n" << flush;
				Output << "Attempting to recover from deadlock...\n";
					
				//**************************************************
				// Go through the list of deadlocked trains until a 
				// train is found that can back up one instruction.
				//**************************************************
				for(int i=0; i<deadcount; i++){
					
					//***************************************************
					// Get a pointer to the current train in the array of
					// deadlocked trains
					//***************************************************
					Train *r;
					r = BlockedTrains.top->link;
					while (r->Id != deadids[i])
						r = r->link;

					if (r->GetMode() ==  BATCH){
										
						int pc;
						int dir;
						int to;
						string instruction;

						//print the trains information
						TrainPrint(r);
						//print the photocell and turnout manager to resource log
						ResourceTablesPrint();
						//print the table of yellow lights
						DisplayYellow();

						r->BlockInstructions = BLOCK; //Block trains from any instructions

						if(r->PhotocellNext == r->DepartedPC){
							//Slide Past the Photocell
							Output << "\nTrain " << r->GetId() << " slid past pc " 
								   << r->PhotocellNext << endl;
							//Back up 1 instruction
							if(r->cursor == 0)
								r->cursor = r->LinesRead;
							else
								r->cursor --;
						}//End Slid Past Photocell
						
						else{
						//Covering the Photocell
							Output << "\nTrain " << r->GetId() << " is covering pc " 
								   << r->PhotocellNext << endl;
							//Back up 2 instructions
							if(r->cursor == 1)
								r->cursor = r->LinesRead;
							else if (r->cursor == 0)
								r->cursor = r->LinesRead - 1;
							else
								r->cursor -=2;
						}

						//read info for trains next resources and instruction
						r->ReadInstruction(pc, dir, instruction);

						//get the next TO
						if (r->GetDirectionAroundTrack() == CLOCKWISE)
							to = CC[r->PhotocellNext].FindToNext();
						else
							to = CW[r->PhotocellNext].FindToNext();

						if(NumAtYellow[pc] > 1){
							//*************************************************
							//There is at least one other train that has also  
							//marked this section of track with yellow lights.  
							//Leave train blocked until the other train(s) have  
							//deallocated their yellow lights from this PC
							//*************************************************
							
							//*************************************************
							//move the instruction pointer for this train back 
							//to the original instruction it was pointing to
							if(r->PhotocellNext != r->DepartedPC){
								//Train is covering the PC
								if(r->cursor == r->LinesRead)
									r->cursor = 0;
								else
									r->cursor ++;
							}
							//*************************************************

							Output << "\nTrain " << r->Id 
								   << " is blocked by a yellow light at photocell " << pc  
								   << "\nPhotocell " << pc << " and Turnout " << to 
								   << " were NOT allocated to train " << r->Id << ".\n";
							
							TrainPrint(r);
						}
						else if (PhotocellManager[pc] == 0){
							//*************************************************
							//The PC one instruction behind for this train is
							//available.  Execute a handle arrival to move the
							//train back one instruction.
							//*************************************************
							int pc1, pc2, to1;
						
							int x = instruction.find(" ");
							string s = instruction.substr(x, x+3);
							int speed1 = atoi(s.c_str());

							Output << "\nNext Instruction - Train Id: " << r->Id 
								   << "  Pc: " << pc << "  Turnout: " << to 
								   << "  Instr: " << instruction;
							
							//Move the train to the active train list
							BlockedToActive(r); 

							//Deallocate any resources (besides current PC) that 
							//this train holds.
							HandleDeparture(r->PhotocellNext);

							//******************************************************
							//Set up the train variables so that when it changes
							//directions, it will correctly reflect PC has, PC next
							//TO has, and LastPC
							r->PhotocellHas = r->PhotocellNext;
							if (r->GetDirectionAroundTrack() == CLOCKWISE){
								r->TurnoutHas = CW[r->PhotocellHas].FindToNext();
								r->PhotocellNext = CW[r->PhotocellHas].FindPCNext();
								if(r->PhotocellNext == -1)
									r->PhotocellNext = CW[r->PhotocellHas].FindPCNext(TurnoutOpposite(r->TurnoutHas));
							}
							else{
								r->TurnoutHas = CC[r->PhotocellHas].FindToNext();
								r->PhotocellNext = CC[r->PhotocellHas].FindPCNext();
								if(r->PhotocellNext == -1)
									r->PhotocellNext = CC[r->PhotocellHas].FindPCNext(TurnoutOpposite(r->TurnoutHas));
							}
							//******************************************************

							r->ChangeDirection();

							Output << "\nTrain " << r->Id << " has changed directions.";

							//Update train allocations
							r->OldPhotocellHas = r->PhotocellHas;
							r->OldTurnoutHas = r->TurnoutHas;
							r->PhotocellHas = r->PhotocellNext;
							r->PhotocellNext = pc;
							r->TurnoutHas = to;

							PhotocellManager[r->PhotocellHas] = r->Id;  
							//Fill the array in the photocellhas location with 
							//the Trains id
							
							PhotocellManager[r->PhotocellNext] = r->Id;
							//Fill the array in the photocellnext location with 
							//the Trains id
							
							if (to != -1){
								TurnoutMng[to] = r->Id;
								//if a turnout exists fill the array at the location 
								//with the Trains id
							}

							//*************************************************
							//Update yellow light allocations
							if(r->GetDirectionAroundTrack() == CLOCKWISE){
								//one PC ahead in current direction of next TO
								pc1 = CW[pc].FindPCNext(); 

								//one TO ahead
								to1 = CW[pc].FindToNext(); 

								//one PC ahead in opposite direction of next TO
								if(to1 != -1)
									pc2 = CW[pc].FindPCNext(TurnoutOpposite(to1)); 
								else
									pc2 = pc1;
							}
							else{
								//one PC ahead in current direction of next TO
								pc1 = CC[pc].FindPCNext();
								
								//one TO ahead
								to1 = CC[pc].FindToNext(); 
								
								//one PC ahead in opposite direction of next TO
								if(to1 != -1)
									pc2 = CC[pc].FindPCNext(TurnoutOpposite(to1)); 
								else
									pc2 = pc1;
							}
							
							if(pc1 != -1)
								AddToYellow(pc1, r->Id);
							if(pc2 != pc1 && pc2 != -1)
								AddToYellow(pc2, r->Id);
							Output << endl;
							//*************************************************
							
							//change trains speed
							SetSpeed(r->Id, speed1);

							//output to our log file
							Output << "\nTrain " << r->Id << " was allocated photocells: " 
								   << r->PhotocellHas << " and " << pc 
								   << " and turnouts: " << r->TurnoutHas << endl;

							//print the trains information
							TrainPrint(r);
							//print the photocell and turnout manager to resource log
							ResourceTablesPrint();
							DisplayYellow();

							//Change TO has to the correct direction
							BatchToSwitch(r,pc, to);

							//Update the train to reflect that the train has successfully
							//departed the current PC
							r->DepartedPC = r->PhotocellHas;
							
							//Move train to next PC
							r->ExecuteLastDCC_Command();

							Output << "\n...End Deadlock Recovery\n";

							r->BlockedStatus = NOTBLOCK;
							return;
						}

						else{
							//*************************************************
							//The photocell one instruction behind is blocked
							//(allocated) to another train.  Train remains
							//blocked.
							//*************************************************

							//*************************************************
							//move the instruction pointer for this train back 
							//to the original instruction it was pointing to
							if(r->PhotocellNext != r->DepartedPC){
								//Covering Photocell
								if(r->cursor == r->LinesRead)
									r->cursor = 0;
								else
									r->cursor ++;
							}
							//*************************************************

							Output << "\nTrain " << r->Id 
								   << " was blocked by another train at photocell " << pc 
								   << "\nTrain " << r->Id <<" was not allocated photocell:"
								   << pc << " or turnout " << to;

							TrainPrint(r);
						}//end blocked by train at pc
							
					}//End Batch
				}//end for
				p->BlockedStatus = NOTBLOCK;
				Output << "\nLeaving Deadlock!\n";
			}//End Deadlock
			else
				Output << "\n\nDEADLOCK was NOT detected!\n\n" << flush;
			
			p->BlockedStatus = NOTBLOCK;
			return;
			//when we find its place adjust the one before it in the list
			//to point to it and adjust the trains pointer to point to the 
			//next one in the list
			
		}//end if p->GetId() == p->GetId()

	  prev=p; //didn't find it yet keep traveling in the activetrainlist
	  p=p->link;
	}//end while p!=ActiveTrains.top

}//End ActiveToBlocked



/////////////////////////////////////////////////////////////////////////////////
//This function Takes a Train and moves it from the blocked train list to the active
//train list.  It is called when a train can be woken back up because the resources 
// it needs have now become available. 
//////////////////////////////////////////////////////////////////////////////////
void ResourceManager::BlockedToActive(Train * T)
{
	//See documentation for Active to Blocked 
	Train *prev, *p;        

	prev=BlockedTrains.top;
	p=BlockedTrains.top-> link;

	while (p!=BlockedTrains.top) 
	{

		if (p->GetId() == T->GetId())
		{
		    prev-> link =p -> link;
		   
			Train * q, *qprev;
			qprev = ActiveTrains.top;
			q = ActiveTrains.top->link;

			while(p->GetId() > q->GetId())
			{
				qprev = q;
				q = q->link;
			}
			
			p->link = q;
			qprev->link = p;
			p->BlockedStatus = ACTIVE;
			Output << "\nTrain " << p->Id << " was moved to active train list ";
			
			return;
			
		}

	  prev=p;
	  p=p->link;
	}

}

/////////////////////////////////////////////////////////////////////////
//Handles an Arrival for a photocell.  This is called from OnTimer() when a 
//photocells state goes from 0 to 1 signifying an arrival.  Fuction gets
//the id of the train from the photocellmanager.  This photocell must be owned
//by a train as their next photocell so get the id held in photocellmanager
//Get a pointer to the train using Find a TrainList function declared in TrainList.h
//and implemented in TrainList.cpp.  Set the LastPhotoCellCovered to be this photocell
//and set blocking instructions to BLOCK so that a train cannot recieve any instructions
//while over a photocell.  Use the TrackList indexed at inputed photocell to dind
//next photocell and turnout.  Update these resources if possible and if not possible
//stop the train and move it to the blocked list
//
//Added: Spring 2002
//	* Batch processing updated
//      - PC slide by correction
//		- Change direction algorithm
///////////////////////////////////////////////////////////////////////////////////

void ResourceManager::HandleArrival(int photocell)
{
	Output << "\n\nHandleArrival: PC=" << photocell; //BM 2-14-02
	
	int id = PhotocellManager[photocell];  //find the trains id


	Train * T = ActiveTrains.Find(id); // get pointer to train using id

	//no train should have been at this location return and print message
	//in log file
	if (T == NULL)
	{//BM - added number of photocell to output 2-14-02
	   Output << "HandleArrival: Photocell " << photocell 
		      << " covered and no train was found" << endl;
	   return;
	}

	T->LastPhotocellCovered = photocell; //update last photocell covered

	T->BlockInstructions = BLOCK; //Block trains from any instructions

	int pc;
	int dir;
	int to;
	string instruction;
		
	if (T->GetMode() ==  BATCH){

		//get the current direction of the train to check 
		//against the next direction
		int lastdir = T->GetDirection();
		
		//read info for trains next resources and instruction
		T->ReadInstruction(pc, dir, instruction);
		
		if(photocell == T->DepartedPC){
			//Slid Past the Photocell
			Output << "\nTrain " << T->GetId() << " slid past pc " 
				   << photocell << endl;

			//resume last speed
			SetSpeed(T->GetId(), T->GetSpeed());
			
			//output to our log file
			Output << "Train " << T->Id << " is still allocated photocells: " 
				   << T->PhotocellHas << " and " << T->PhotocellNext 
				   << " and turnouts: " << T->TurnoutHas << endl;
			
			//return cursor to original position
			if(T->cursor == 0)
				T->cursor = T->LinesRead;
			else
				T->cursor --;

			//print the trains information
			TrainPrint(T);
			//print the photocell and turnout manager to resource log
			ResourceTablesPrint(); 
			//print yellow light allocation in the resource log
			DisplayYellow();

			T->ExecuteLastDCC_Command();
			T->BlockedStatus = NOTBLOCK;
			return;
		}//End Slid Past Photocell

		//get the next TO
		if (T->GetDirectionAroundTrack() == CLOCKWISE)
			to = CW[photocell].FindToNext();
		else
			to = CC[photocell].FindToNext();

		//output to resource log
		Output << "\nNext Instruction - Train Id: " << T->Id << "  Pc: " << pc 
			   << "  Turnout: " << to << "  Instr: " << instruction;
		
		int x = instruction.find(" ");
		string s = instruction.substr(x, x+3);
		int speed1 = atoi(s.c_str());

		//change trains speed
		SetSpeed(T->Id, speed1);
			
		if(lastdir != dir){
			//*****************************************************************
			//This instruction has requested a change in the trains current
			//direction.  Change the train's direction, move the instruction
			//pointer back one instruction, then issue another handle arrival
			//that will process the instruction in the correct direction.
			//*****************************************************************

			int pc1, pc2, to1, removed;

			//*****************************************************************
			//Remove the yellow light allocations for the train in its current
			//direction before the direction is changed.
			if (T->GetDirectionAroundTrack() == CLOCKWISE){
				pc1 = CW[T->PhotocellNext].FindPCNext();
				to1 = CW[T->PhotocellNext].FindToNext();
				if(to1 != -1)
					pc2 = CW[T->PhotocellNext].FindPCNext(TurnoutOpposite(to1));
				else
					pc2 = -1;
			}
			else{
				pc1 = CC[T->PhotocellNext].FindPCNext();
				to1 = CC[T->PhotocellNext].FindToNext();
				if(to1 != -1)
					pc2 = CC[T->PhotocellNext].FindPCNext(TurnoutOpposite(to1));
				else
					pc2 = -1;
			}

			removed = 0;
			if(pc1 != -1)
				removed = RemoveFromYellow(pc1, T->Id);

			//*****************************************************************
			// If a train was blocked due to a cautious yellow light (i.e. a PC  
			// that was marked with a yellow light that the train never  
			// actually traveled to) and can now be returned to the active    
			// train list, call ReleaseBlockedTrains.  
			//
			// NOTE: A -2 was used as the turnout number because there is no  
			// turnout -2, thus a train will only be moved to active if it was  
			// blocked by the yellow light on this PC
			//*****************************************************************
			if(removed && pc1 != pc){
				ReleaseBlockedTrains(pc1, -2);
				Output << "\nTrain Id: " << T->Id 
					   << " has released blocked trains at pc " << pc1;
			}

			removed = 0;
			if(pc1 != pc2 && pc2 != -1)
				removed = RemoveFromYellow(pc2, T->Id);

			//*****************************************************************
			// If a train was blocked due to a cautious yellow light (i.e. a PC  
			// that was marked with a yellow light that the train never  
			// actually traveled to) and can now be returned to the active    
			// train list, call ReleaseBlockedTrains.  
			//
			// NOTE: A -2 was used as the turnout number because there is no  
			// turnout -2, thus a train will only be moved to active if it was  
			// blocked by the yellow light on this PC
			//*****************************************************************
			if(removed && pc2 != pc){
				ReleaseBlockedTrains(pc2, -2);
				Output << "\nTrain Id: " << T->Id 
					   << " has released blocked trains at pc " << pc2;
			}
			//*****************************************************************

			//Deallocate any resources (besides current PC) that 
			//this train holds.
			HandleDeparture(photocell);

			//******************************************************
			//Set up the train variables so that when it changes
			//directions, it will correctly reflect PC has, PC next
			//TO has, and LastPC
			T->PhotocellHas = T->PhotocellNext;
			if (T->GetDirectionAroundTrack() == CLOCKWISE){
				T->TurnoutHas = CW[T->PhotocellHas].FindToNext();
				T->PhotocellNext = CW[T->PhotocellHas].FindPCNext();
				if(T->PhotocellNext == -1)
					T->PhotocellNext = CW[T->PhotocellHas].FindPCNext(TurnoutOpposite(T->TurnoutHas));
			}
			else{
				T->TurnoutHas = CC[T->PhotocellHas].FindToNext();
				T->PhotocellNext = CC[T->PhotocellHas].FindPCNext();
				if(T->PhotocellNext == -1)
					T->PhotocellNext = CC[T->PhotocellHas].FindPCNext(TurnoutOpposite(T->TurnoutHas));
			}
			//*******************************************************

			//Change the train's direction - function found in train.cpp
			T->ChangeDirection();

			//move cursor back to this instruction
			if(T->cursor == 0)
				T->cursor = T->LinesRead;
			else
				T->cursor --;

			//Call handle arrival on this photocell now that the train
			//is in the correct direction.
			HandleArrival(photocell);

			T->BlockedStatus = NOTBLOCK;
			return;
		}//End Switch Direction

	}//End Batch

	//find the next photocell and turnout using TrackList Function
	else if (T->GetDirectionAroundTrack() == CLOCKWISE)
	{
		pc = CW[photocell].FindPCNext();
		to = CW[photocell].FindToNext();
	}

	else if (T->GetDirectionAroundTrack() == COUNTERCLOCKWISE)
	{
		pc = CC[photocell].FindPCNext();
		to = CC[photocell].FindToNext();
	}


///////////////////////////////////////////////////////////////////////////
//got our data about next resrouces now lets see if we can actually give
//the trains these resources
///////////////////////////////////////////////////////////////////////////

	//Deallocate any resources (besides current PC) that this train holds.
	HandleDeparture(photocell);

	if (pc != -1)
	{
		if(NumAtYellow[pc] > 1){
			//*************************************************
			//UNSAFE state!
			//
			//There is at least one other train that has also  
			//marked this section of track with yellow lights.  
			//Block train until the other train(s) have  
			//deallocated their yellow lights from this PC.
			//*************************************************

			//block train
			T->BlockingPhotocell = pc;
			T->BlockingTurnout = to;
			T->Stop();
			Output << "Train " << T->Id 
				   << " reached an UNSAFE state on the track." << endl 
				   << "Photocell " << pc << " and Turnout " << to 
				   << " were NOT allocated.\n";

			//print critical information to resource log
			TrainPrint(T);
			ActiveToBlocked(T); 
			DisplayYellow();
		}
		else {
			//if pc is not -1 we can continue travel in this direction
			//if we can allocat the resources
			if (UpdateResources(T,pc,to)) //update resources if possible
			{
				T->DepartedPC = photocell;
				T->ExecuteLastDCC_Command(); 
				//execute last command in case we are waking up train from
				//being blocked
			}

			else
			{
				//if not possible block train
				T->BlockingPhotocell = pc;
				T->BlockingTurnout = to;
				T->Stop();
				Output << "Train " << T->Id << " couldn't be allocated photocell:"
					   << pc << " and turnout " << to;
				TrainPrint(T);
				ActiveToBlocked(T);
				DisplayYellow();
			}
		}

	}	
	
	//pc is -1 continued travel is impossible
	//toggle turnout to make travel possible
	else{

		//refind the next photocell using opposite of current turnout position
		if (T->GetDirectionAroundTrack() == CLOCKWISE)
			pc = CW[photocell].FindPCNext(TurnoutOpposite(to));
		else if (T->GetDirectionAroundTrack() == COUNTERCLOCKWISE)
			pc = CC[photocell].FindPCNext(TurnoutOpposite(to));
		


		ToggleTurnout(to); 
		//Actually toggle the turnout use the resource manager function so that 
		//we don't toggle it if it is already owned since we haven't 
		//checked yet

		if(NumAtYellow[T->PhotocellNext] > 1){
			//*************************************************
			//UNSAFE state!
			//
			//There is at least one other train that has also  
			//marked this section of track with yellow lights.  
			//Block train until the other train(s) have  
			//deallocated their yellow lights from this PC.
			//*************************************************

			//block train
			T->BlockingPhotocell = pc;
			T->BlockingTurnout = to;
			T->Stop();
			
			//print critical information to the resource log
			Output << "Train " << T->Id << " couldn't be allocated photocell:"
				   << pc << " and turnout " << to;
			
			TrainPrint(T);
			ActiveToBlocked(T); 
			DisplayYellow();
		}
		else {
			//if pc is not -1 we can continue travel in this direction
			//if we can allocat the resources
			if (UpdateResources(T,pc,to)) //update resources if possible
			{
				T->DepartedPC = photocell;
				T->ExecuteLastDCC_Command(); 
				//execute last command in case we are waking up train from
				//being blocked
			}

			else
			{
				//if not possible block train
				T->BlockingPhotocell = pc;
				T->BlockingTurnout = to;
				T->Stop();
				Output << "Train " << T->Id << " couldn't be allocated photocell:"
					   << pc << " and turnout " << to;
				TrainPrint(T);
				ActiveToBlocked(T);
			}
		}
	}
	T->BlockedStatus = NOTBLOCK;
}



//////////////////////////////////////////////////////////////////
//This function finds the train causing a departure and calls
//DeallocateResources to actually deallocate the trains resources
//set the trains BlockedInstructions back to NOTBLOCK so that
//the train can change direction again
//
//Spring 2002
//	This function is no longer called from the function OnTimer(),
//	rather it is called when ever an arrival is handled.
///////////////////////////////////////////////////////////////////
void ResourceManager::HandleDeparture(int photocell)
{
	//find the train handleing departure
	int id = PhotocellManager[photocell];

	Train * T = ActiveTrains.Find(id);

	//if no train owned this photcell return must have been a shadow
	if (T == NULL)
		{//BM - added number of photocell to output 2-14-02
		   Output << "\n\nHandleDeparture: Photocell " << photocell 
					<< " covered and no train was found" << endl;
		   return;
		}

	Output << "\n\nHandleDeparture: PC=" << T->PhotocellHas ; //BM 2-14-02

	//resource log output
	Output << "\nTrain " << T->Id << " deallocating resources" ;

	DeallocateResources(T); //actually deallocate the resources

	T->BlockInstructions = NOTBLOCK;

	//set train variable so that can change direction
	Output << "\nCompleted HandleDeparture: PC=" << T->PhotocellHas; //BM 2-14-02

}



/////////////////////////////////////////////////////////////////////
//Print the contents of a train to the resource log
///////////////////////////////////////////////////////////////////
void ResourceManager::TrainPrint(Train * T)
{
	char str[10];

	//*************************************************************************
	//bmcvey  3-25-02  altered spacing on name, made str have correct
	//				   label rather than displaying 0 or 1 (forward/reverse,etc)
	//*************************************************************************

	if (T->BlockedStatus == BLOCKED)
		strcpy(str, "Blocked");
	else
		strcpy(str, "Active");

	Output << endl
		 << setw(17) << " Train id: " << setw(3) << T->Id
		 << setw(20) << " BlockedStatus: " << setw(3) << str
		 << setw(17) << " Name: " << T->Name << endl;

	if (T->DirectionAroundTrack == CLOCKWISE)
		strcpy(str, "CW");
	else
		strcpy(str, "CC");

	Output << setw(17) << " PChas: " << setw(3) << T->PhotocellHas
		 << setw(20) << " Blocking PC: " << setw(3) << T->BlockingPhotocell
		 << setw(20) << " TrackDir: " << setw(3) << str << endl;

	if (T->Direction == FORWARD)
		strcpy(str, "Forward");
	else
		strcpy(str, "Reverse");

	Output << setw(17) << " PCnext: " << setw(3) << T->PhotocellNext 
		 << setw(20) << " Blocking TO: " << setw(3) << T->BlockingTurnout 
		 << setw(20) << " TrainDir: "<< setw(3) << str  << endl;

	if (T->HeadLight == ON)
		strcpy(str, "On");
	else
		strcpy(str, "Off");

	Output << setw(17) << " OldPChas: " << setw(3) << T->OldPhotocellHas
		 << setw(20) << " Blocking Instr: " << setw(3) << T->BlockInstructions 
		 << setw(20) << " Headlight: " << setw(3) << str << endl;

	if (T->TailLight == ON)
		strcpy(str, "On");
	else
		strcpy(str, "Off");

	Output << setw(17) << " LastPC covered: " << setw(3) << T->LastPhotocellCovered 
		 << setw(20)<< "  CurSpeed: " << setw(3) << T->Speed
		 << setw(20) << " TailLight: " << setw(3) << str << endl;

	if (T->Mode == BATCH)
		strcpy(str, "Batch");
	else
		strcpy(str, "Inter");
	Output << setw(17) << " TOhas: " << setw(3) << T->TurnoutHas
	     << setw(20)<< "  MaxSpeed: " << setw(3) << T->MaxSpeed  
	     << setw(20) << " Mode: " <<  setw(3) << str  << endl;

	Output << setw(17) << " OldTOhas: " << setw(3) << T->OldTurnoutHas 
	     << setw(20)<< " Last speed: " << setw(3) << T->LastUserSpeed 
	     << setw(20) << " DepartedPC: " << T->DepartedPC << endl << endl;
	   
}



/////////////////////////////////////////////////////////////////////
//Searches the Blocked list to see if the inputed photocell number is
//the photocell that is blocking any of the trains on the blocked list
//if it finds a trian that is blocked by this resource it will release
//that train from the blocked list and move it to the activelist
//it will also reissue an arrival to the trains last covered photocell
//and a departure if the train is no longer covering that photocell to get 
//the correct values in the photocell and turnoutmng tables
//
//Added: Spring 2002
//		 A train will only be added back to the active list if its 
//		 blocking PC/TO has been released AND there are no trains
//		 that have allocated that section of track will yellow lights.
///////////////////////////////////////////////////////////////////////
void ResourceManager::ReleaseBlockedTrains(int ReleasedPhotocell, int ReleasedTurnout)
{

	Train *p;          // used for tag along

	p=BlockedTrains.top-> link;

	while (p!=BlockedTrains.top)
	{

		//we found a train that can be woken up
		if ((ReleasedPhotocell == p->BlockingPhotocell || 
			ReleasedTurnout == p->BlockingTurnout) &&
			NumAtYellow[p->BlockingPhotocell] == 0)
		{
			//move it to the active lisdt
			BlockedToActive(p);

			//handle the arrival again to fake the arrival again 
			HandleArrival(p->LastPhotocellCovered);

			return;
		}

	     p=p->link; //continue through the blocked list
	}

}


/////////////////////////////////////////////////////////////
//returns the Current allocation of a particular photocell
//returns 0 if not allocated.  Returns Trains id that ownes the
// resource if it is allocated
///////////////////////////////////////////////////////////
int ResourceManager::GetPhotoCellManager(int index)
{
	return PhotocellManager[index]; //returns allocation of photocellmanager
}

///////////////////////////////////////////////////////////////////////////
//returns the current allocation of a particular photocell
//returns 0 if not allocated. Returns Train's id that owns the resource if
//the resource is allocated
///////////////////////////////////////////////////////////////////////////
int ResourceManager::GetTurnoutManager(int index)
{
	return TurnoutMng[index];
}


//////////////////////////////////////////////////////////////////////////
//Toggles a specified turnout on the track if that turnout is not allocated
//to any train.  Calls the TO_Toggle(to) function defined in ccr.h and implemented
//in ccr.dll which is pointed to by the dp.lib
//Returns 0 if successful or the result of TO_Toggle(to) if successful
///////////////////////////////////////////////////////////////////////////////////
int ResourceManager::ToggleTurnout(int to)
{

	if (TurnoutMng[to] != 0) 
	//check to make sure that toggling this turnout won't make a train fly off
	//the track because it is in use by a train by making sure it isn't allocated
	//to any train
	{
		Output << "User attempted to toggle turnout when it was allocated to train " 
			   << TurnoutMng[to] << endl;
		return 0; // not possible
	}

	int todir = TurnoutOpposite(to);

	char dir[15];

	if (todir == STRAIGHT)
		strcpy(dir, "straight");
	else
		strcpy(dir, "curved");

	int success = TO_Toggle(to);

	if (success)
		Output << "Toggled turnout " << to << " to " << dir << endl;

	return success;

	
}

/////////////////////////////////////////////////////////////////////////////////
//This function returns the opposite state of a turnout.  If the result of 
//Get_TO_Status(to) is STRAIGHT this fuction return CURVED.  If it is CURVED
//this returns STRAIGHT.  If neither it returns that the turnout is MOVING
//Get_TO_Status(int to) is defined in ccr.h
///////////////////////////////////////////////////////////////////////////

int ResourceManager::TurnoutOpposite(int to){


	if (Get_TO_Status(to) == STRAIGHT)
		return CURVED;
	else if (Get_TO_Status(to) == CURVED)
		return STRAIGHT;
	else 
		return MOVING;
}



/////////////////////////Resource Train Functions///////////////////////////////////


////////////////////////////////////////////////////////////////////////////
//The following functions are wrapper functions that call a trains methods
//ie performing motor functions like seting speed or changing direction, toggling
//headlights and taillights. 
//////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////
//This Function Sets the speed of a train with the inputed id number  
//It finds the train in the active list checks to see if instructions are
//blocked to that train (trains are not allowed to get instructions when over
//a photocell between allocation and deallocation because they are under operating
//system control---Don't want them to stop or change direction while over a 
//photocell -- returns the value of SetSpeed()-- a train method defined in Train.h
//and implemented in train.cpp
//////////////////////////////////////////////////////////////////////////////////

int ResourceManager::SetSpeed(int id, int newspeed)
{



	Train * T = ActiveTrains.Find(id); 
	//find the train with this id

	if (T == NULL)
		return 0;

	int success = T->SetSpeed(newspeed); 

	if (success)
	{
		T->LastUserSpeed = T->Speed;
		Output << "\nTrain " << T->Id << " speed changed--all propertys follow";
	}
	return success;
	// else perform SetSpeed() and return its value

}


//////////////////////////////////////////////////////////////////////////
//This Function stops a train  with the inputed id number  
//It finds the train in the active list checks to see if instructions are
//blocked to that train (trains are not allowed to get instructions when over
//a photocell between allocation and deallocation because they are under operating
//system control---Don't want them to stop or change direction while over a 
//photocell -- returns the value of Stop()-- a train method defined in Train.h
//and implemented in train.cpp
//////////////////////////////////////////////////////////////////////////////////

int ResourceManager::Stop(int id)
{
	Train * T = ActiveTrains.Find(id);
	//find the train with this id

	if (T == NULL)
		return 0;

	if (T->BlockInstructions == BLOCK)
		return 0;
	// if we are in between allocation and deallocation signified by BlockingInstruction
	// being BLOCK return without doing anything

	int success = T->Stop();
	if (success)
	{
		Output << "Train " << T->Id << " stoped--all propertys follow" << endl;
	    TrainPrint(T);
	}
	return success;
	// else perform Stop() and return its value
}



//////////////////////////////////////////////////////////////////////////
//This Function changes the direction of a train  with the inputed id number  
//It finds the train in the active list checks to see if instructions are
//blocked to that train (trains are not allowed to get instructions when over
//a photocell between allocation and deallocation because they are under operating
//system control---Don't want them to stop or change direction while over a 
//photocell -- returns the value of ChangeDirection()-- a train method defined in Train.h
//and implemented in train.cpp
//////////////////////////////////////////////////////////////////////////////////
int ResourceManager::ChangeDirection(int id)
{

	Train * T = ActiveTrains.Find(id);
	//find the train with this id

	if (T == NULL)
		return 0;

	if (T->BlockInstructions == BLOCK)
		return 0;
	// if we are in between allocation and deallocation signified by BlockingInstruction
	// being BLOCK return without doing anything

	int success = T->ChangeDirection();
	if (success)
	{
		Output << "Train " << T->Id << " changed direction--all propertys follow" << endl;
	    TrainPrint(T);
	}
	return success;	
	// else perform ChangeDirection() and return its value

}



//////////////////////////////////////////////////////////////////////////
//This Function turns the headlight of a train to the on or off position
//It finds the train in the active list checks to see if instructions are
//blocked to that train (trains are not allowed to get instructions when over
//a photocell between allocation and deallocation because they are under operating
//system control---Don't want them to stop or change direction while over a 
//photocell -- returns the value of TurnHeadLightOn()-- a train method defined in Train.h
//and implemented in train.cpp
//////////////////////////////////////////////////////////////////////////////////
int ResourceManager::TurnHeadLightOn(int id, int on)
{

	Train * T = ActiveTrains.Find(id);
	//find the train with this id

	if (T == NULL)
		return 0;

	if (T->BlockInstructions == BLOCK)
		return 0;
	// if we are in between allocation and deallocation signified by BlockingInstruction
	// being BLOCK return without doing anything

	int success = T->TurnHeadLightOn(on);
	if (success)
	{
		Output << "Train " << T->Id << " HeadLight changed--all propertys follow" << endl;
	    TrainPrint(T);
	}
	return success;	
	// else perform TurnoutHeadLightOn() and return its value


}


//////////////////////////////////////////////////////////////////////////
//This Function toggles the headlight of a train 
//It finds the train in the active list checks to see if instructions are
//blocked to that train (trains are not allowed to get instructions when over
//a photocell between allocation and deallocation because they are under operating
//system control---Don't want them to stop or change direction while over a 
//photocell -- returns the value of ToggleHeadLight()-- a train method defined in Train.h
//and implemented in train.cpp
//////////////////////////////////////////////////////////////////////////////////

int ResourceManager::ToggleHeadLight(int id)
{
	Train * T = ActiveTrains.Find(id);
	//find the train with this id

	if (T == NULL)
		return 0;

	if (T->BlockInstructions == BLOCK)
		return 0;
	// if we are in between allocation and deallocation signified by BlockingInstruction
	// being BLOCK return without doing anything


	int success = T->ToggleHeadLight();
	if (success)
	{
		Output << "Train " << T->Id << " Headlight toggled -all propertys follow" << endl;
	    TrainPrint(T);
	}
	return success;	
	// else perform ToggleHeadLight() and return its value
}


//////////////////////////////////////////////////////////////////////////
//This Function turns the taillight of a train to the on or off position
//It finds the train in the active list checks to see if instructions are
//blocked to that train (trains are not allowed to get instructions when over
//a photocell between allocation and deallocation because they are under operating
//system control---Don't want them to stop or change direction while over a 
//photocell -- returns the value of TurnTailLightOn()-- a train method defined in Train.h
//and implemented in train.cpp
//////////////////////////////////////////////////////////////////////////////////
int ResourceManager::TurnTailLightOn(int id, int on)
{

	Train * T = ActiveTrains.Find(id);
	//find the train with this id

	if (T == NULL)
		return 0;

	if (T->BlockInstructions == BLOCK)
		return 0;
	// if we are in between allocation and deallocation signified by BlockingInstruction
	// being BLOCK return without doing anything

	int success = T->TurnTailLightOn(on);
	if (success)
	{
		Output << "Train " << T->Id << " TailLight changed -all propertys follow" << endl;
	    TrainPrint(T);
	}
	return success;
	// else perform TurnTailLightOn() and return its value

}



//////////////////////////////////////////////////////////////////////////
//This Function toggles the Taillight of a train 
//It finds the train in the active list checks to see if instructions are
//blocked to that train (trains are not allowed to get instructions when over
//a photocell between allocation and deallocation because they are under operating
//system control---Don't want them to stop or change direction while over a 
//photocell -- returns the value of ToggleTailLight()-- a train method defined in Train.h
//and implemented in train.cpp
//////////////////////////////////////////////////////////////////////////////////
int ResourceManager::ToggleTailLight(int id)
{
	Train * T = ActiveTrains.Find(id);
	//find the train with this id

	if (T == NULL)
		return 0;

	if (T->BlockInstructions == BLOCK)
		return 0;
	// if we are in between allocation and deallocation signified by BlockingInstruction
	// being BLOCK return without doing anything

	int success = T->ToggleTailLight();
	if (success)
	{
		Output << "Train " << T->Id << " taillight toggled--all propertys follow" << endl;
	    TrainPrint(T);
	}
	return success;
	// else perform ToggleTailLight() and return its value

}


////////////////////////////////////////////////////////////////////////////
//This function toggles the course status from enabled to disabled or disabled to
//enables by issuing the DCC_reset() or DCC_kill() commands.
//If the function kills the track it stops all trains before issuing the kill command
//and toggles headlights and taillights to off. If the function enables the track
//it will reissue the last user speed command to all the trains on the active list
//////////////////////////////////////////////////////////////////////////////////
void ResourceManager::ToggleEnabledCourse()
{


	if (TrackEnabled)  // if the track was enabled
	{
		//output to log file
		Output << "Track Disabled by User" << endl;
		//set the bool to see we are no longer opened for business
		TrackEnabled= false;

		Train *p;          // used for tag along

		p=ActiveTrains.top-> link;

		//search the active list
		while (p!=ActiveTrains.top)
		{
			p->Stop(); //and stop all trains
			p->TurnHeadLightOn(OFF); //turnout off the headlights
			p->TurnTailLightOn(OFF); //and taillights
		    p=p->link;
		}

		MyDCC_kill(); //kill the track

	}
	else
	{
		//output to log file
		Output << "Track Enabled by User" << endl;

		//adjust the bool to now true
		TrackEnabled = true;

		//actually reset the track to get commands again
		MyDCC_reset();
			
		Train *p;          // used for tag along

		p=ActiveTrains.top-> link; 
		//search the active list
		while (p!=ActiveTrains.top)
		{
			p->ExecuteLastDCC_Command(); //giving all trains their last speed
			p=p->link;
		}

	}


}

/////////////////////////End Resource Train Functions///////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////
//Loads the values into the array of ListLists CC and CW.  These data structures
//act as functions which output the next photocell and turnout that is associated
//with a photocell.  For example if CW[0] has a node (2, 1, STRAIGHT) it means that 
//if a train is heading Clockwise and hits photocell 0 then photocell 2 is connected
//to phtocell 0 through turnout 1 if turnout 1 is in a straight direction.
//Muliple nodes are added to the same index of the array when turnouts make it possible
//to travel to two different photocells.  In this case two separate nodes are added
//to the same index in CC or CW
///////////////////////////////////////////////////////////////////////////////////

void ResourceManager::LoadTrack(){


	//define the Track in a clockwise direction
	CW[0].Add(2,1,STRAIGHT);
	CW[1].Add(2,1,CURVED);
	CW[2].Add(7,-1,NOTURNOUT);
	CW[3].Add(1,0,STRAIGHT);
	CW[4].Add(12,6,MOVING);
	CW[5].Add(4,-1,NOTURNOUT);
//	CW[6].Add(12,-1,NOTURNOUT);
	CW[7].Add(15,2,STRAIGHT);
	CW[8].Add(0,-1,NOTURNOUT);
	CW[9].Add(3,-1,NOTURNOUT);
//	CW[10].Add(-1,-1,NOTURNOUT);
	CW[11].Add(5,6,MOVING);
	CW[12].Add(15,2,CURVED);
	CW[13].Add(8,4,STRAIGHT);
	CW[13].Add(9,4,CURVED);
	CW[14].Add(11,-1,NOTURNOUT);
	CW[15].Add(13,3,STRAIGHT);
	CW[15].Add(14,3,CURVED);


	//define the track in a counterclockwise direction
	CC[0].Add(8,-1,NOTURNOUT);
	CC[1].Add(3,0,STRAIGHT);
	CC[2].Add(1,1,CURVED);
	CC[2].Add(0,1,STRAIGHT);
	CC[3].Add(9,-1,NOTURNOUT);
	CC[4].Add(5,-1,NOTURNOUT);
	CC[5].Add(11,6,MOVING);
//	CC[6].Add(5,-1,NOTURNOUT);
	CC[7].Add(2,-1,STRAIGHT);
	CC[8].Add(13,4,STRAIGHT);
	CC[9].Add(13,4,CURVED);
//	CC[10].Add(-1,-1,-1,NOTURNOUT);
	CC[11].Add(14,-1,NOTURNOUT);
	CC[12].Add(4,6,MOVING);
	CC[13].Add(15,3,STRAIGHT);
	CC[14].Add(15,3,CURVED);
	CC[15].Add(7,2,STRAIGHT);
	CC[15].Add(12,2,CURVED);

}


//******************************************************************
// PURPOSE: The purpose of this function is to deallocate a train  
//          in the yellow light data structure.
//			
// INPUT: The PC to be deallocated and the train id of the train
//		  that it is currently allocated to.
//
// OUTPUT: 0 if the PC was NOT deallocated
//		   1 if the PC was deallocated
//
// DATE: Spring 2002
//******************************************************************
int ResourceManager::RemoveFromYellow(int pc, int id){
	int removed = 0;
	int i=0;
	
	//find the id number in the PC's list of train allocations
	while(i < NumAtYellow[pc] && YellowLights[i][pc] != id) i++;


	if (YellowLights[i][pc] == id){ 
		//The train id number was found.  Move all other train id's
		//up in the list of yellow light allocations for this PC
		for(int j = i + 1; j < NumAtYellow[pc]; j++){
			YellowLights[i][pc] = YellowLights[j][pc];
			i++;
		}
		
		//Replace the last train id with a -1
		YellowLights[i][pc] = -1;

		//Decrease the number of trains with yellow light allocations
		//at this PC
		NumAtYellow[pc] --;

		//The train was successfully removed from the yellow light
		//data structure.
		removed = 1;

		//Output deallocation to the log file
		Output << "\nTrain Id: " << id << " deallocated photocell " 
			   << pc << " in the yellow list.";
	}
	return removed;
}


//******************************************************************
// PURPOSE: The purpose of this function is to allocate a train  
//          in the yellow light data structure.
//			
// INPUT: The PC to be allocated and the train id of the train
//		  wishing allocation
//
// DATE: Spring 2002
//******************************************************************
void ResourceManager::AddToYellow(int pc, int id){
	int found = 0;

	//Check to see if the train has already allocated a yellow 
	//light at this PC.
	for (int i = 0; i < NumAtYellow[pc]; i++){
		if(YellowLights[i][pc] == id)
			found = 1;
	}

	if(!found){
		//Train is not in the list, so add it
		YellowLights[NumAtYellow[pc]][pc] = id;

		//Increase the number of trains that have yellow light
		//allocations at this pc
		NumAtYellow[pc] ++;
		
		//Output allocation to the log file
		Output << "\nTrain Id: " << id << " allocated photocell " 
			   << pc << " in the yellow list.";
	}
	return;
}


//******************************************************************
// PURPOSE: The purpose of this function is to display the   
//          allocation of yellow lights to the resouce log.
//
// DATE: Spring 2002
//******************************************************************
void ResourceManager::DisplayYellow(){
	
	Output << endl << setw(10) << "Photocell" << setw(12) << "Number of" << setw(15) 
		   << "Train Id(s)" << endl << setw(20) << "Trains" << endl;
	for(int i = 0; i < MAXPHOTOCELLS; i++){
		Output << setw(6) << i << setw(12) << NumAtYellow[i] << setw(8) << " ";
		for(int j = 0; j < NumAtYellow[i]; j++){
			Output << setw(3) << YellowLights[j][i];
		}
		Output << endl;
	}
}


//////////////////////////////ResourceManagerWindow Fuctions////////////////////////



int ResourceManager::OnCreate(LPCREATESTRUCT lpcs){


	if (CWnd::OnCreate(lpcs) == -1)
		return -1; //return if not good


	m_font.CreatePointFont(80, _T("MS Sans Serif"));  //select our font
	CClientDC dc (this);  //get handler to compute text averages
	CFont * pOldFont = dc.SelectObject(&m_font); // select our font

	TEXTMETRIC Tm; // get Textmetric

	dc.GetTextMetrics(&Tm);  // and give it our font to calculate
	m_cxChar = Tm.tmAveCharWidth;  // store average width
	m_cyChar = Tm.tmHeight + Tm.tmExternalLeading;	// and height

	dc.SelectObject(pOldFont); // return old pen to not consume resources

	return 0;


}//Oncreate



/////////////////////////////////////////////////////////////////////
//When the window is Destroyed kill the timer we created during the constructor
//of this class
/////////////////////////////////////////////////////////////////////////////
afx_msg void ResourceManager::OnDestroy()
{

     
	KillTimer( 1 );   //kill timer 1

}


//////////////////////////////////////////////////////////////////////////////
//OnTimer is called everytime the timer set in the constructor of this class is
//fired or every half second.  This function keeps track of what photocells have
//changed states. (ie covered to uncovered and uncovered to covered) it then performs
//the operations to handle a trains arrival over that photocell or handle its departure 
//to a photocell.  Starts with a call to UpdateCurrentPCStatus() which reads all current
//values of the photocell states.  Next runs a loop to check if these new values
//are different from the values the last time the timer fired.  If CurrentState is greater
//then we had an arrival since covered is 1 and uncovered is 0.  If the PreviousPCState
//is greater we had a departture and the function calls HandleArrival() or HandleDeparture()
//to handle these situations ---- it then sets the PreviousPCState to be the 
//CurrentPCState to ready itself for the next call
//
//Added: Spring 2002
//		 HandleDeparture() no longer called from this function - see note in
//		 HandleDeparture comments
/////////////////////////////////////////////////////////////////////////////////
afx_msg void ResourceManager::OnTimer()
{

	UpdateCurrentPCStatus(); 
	//updates the array that keeps track of the states(covered, uncovered) of photocells

	for (int i = 0; i<MAXPHOTOCELLS; i++)
	//run through all the photocells
	{

		// if Current of this photocell is larger then it just got covered
		if (CurrentPCState[i] > PreviousPCState[i])
		{
			HandleArrival(i);
			// handle the arrival or this photocell
		}


		// if the Current is less then the Previous the photocell just got
		// uncovered]
		/*
		else if (CurrentPCState[i] < PreviousPCState[i])
		{
			HandleDeparture(i); 
			//Handle the departure of this photocell
		}
		*/

		PreviousPCState[i] = CurrentPCState[i];
		//Update the state of Previous

	}
	   

	Invalidate(FALSE); 
	//forces call to OnPaint when user wants to watch the 
	//resources being allocated for debugging purposes

}



//////////////////////////////End ResourceManagerWindow Fuctions/////////////////////////




/////////////////////////////////////////////////////////////////////////////////
//Returns the Status of a photocell
//Covered = 1
//Uncovered = 0
//////////////////////////////////////////////////////////////////////////////////
int ResourceManager::GetPhotoCellStatus(int pc)
{
	return CurrentPCState[pc];
}


////////////////////////////////////////////////////////////////////////////////
//Returns the state of the inputed turnout -- wraps functions in ccr.dll
/////////////////////////////////////////////////////////////////////////////
int ResourceManager::GetTurnoutDirection(int to)
{

	return (Get_TO_Status(to));

}

//////////////////////////////////////////////////////////////////////////////
//Returns the TrackEnabled Bool
////////////////////////////////////////////////////////////////////////////////
bool ResourceManager::GetTrackEnabled()
{

	return TrackEnabled;


}
/////////////////////////////////////////////////////////////////////////////////
//Updates the array CurrentPCState with the current values read from the photocell
//board
/////////////////////////////////////////////////////////////////////////////
void ResourceManager::UpdateCurrentPCStatus()
{
	for (int i = 0; i< MAXPHOTOCELLS; i++)
		CurrentPCState[i] = Get_Sensor_Status(i);
	
}



BEGIN_MESSAGE_MAP(ResourceManager, CFrameWnd)
	
	ON_WM_CREATE()

	ON_WM_TIMER()    

	ON_WM_PAINT()   


END_MESSAGE_MAP()


  