#include "TrackList.h"
#include "ccr.h"
#include <fstream>
using namespace std;





TrackList::TrackList() {
/////////////////////////////////////////////////
//   Create a new TrackList
////////////////////////////////////////////////
   top = new Node(-1,-1, -1); //header node

   if (top==NULL)
   {
     //out of heap space
   }
   top->link = top;     
   // circle header TrackList


} // TrackList


//////////////////////////////////////////////////////////////////////////////
//Adds a node to the TrackList. Creates a new Node defined in Node.h and adds
//it to the front of the list be making the header node point to it and making
//it point to the old node top used to point to
//Returns 1 if successfully added 0 if not
/////////////////////////////////////////////////////////////////////////////
int TrackList::Add (int pc, int to, int sd)
{
   Node *x;            // points to new node

   x = new Node(pc, to ,sd);

   if (x==NULL)
     return 0;

   x-> link = top->link;
   top-> link = x;
   return 1;

} // Add



////////////////////////////////////////////////////////////////////////////
//	clears all members of the current TrackList
/////////////////////////////////////////////////////////////////////////////
void TrackList::Clear() 
{
   Node *p,*prev;       // for tag along

   prev=top;
   p = top->link;
   top-> link = top;

   while (p != top) {

      // Advance to next item in the TrackList
      prev=p;
      p = p->link;
      delete prev;

   } //while
  
}// end clear


///////////////////////////////////////////////////////////////////////////////
//Given a tracklist this function finds the next photocell giving the current
//track state.  Uses Get_TO_Status(int to) to read the current state of the 
//turnouts to compute what photocell is next. Called from HandleArrival in
//ResourceManager
////////////////////////////////////////////////////////////////////////////
int TrackList::FindPCNext()
{


		Node * p;

	   	p = top->link; // get top of list to start to search
		
		if (p->turnout == -1)
				return (p->photocell);
		//if no turnout is connected to the photocell then only one
		//photocell can be connected in either direction so just return
		//the photocell stated in the first node

		else if (p->switchdirection == Get_TO_Status(p->turnout))
				return(p->photocell);
		//if the curent switch direction read from the board of the turnout matches
		//this one return the photocell in this node
				
		else if (p->link == top)
				return  -1; 
		//if we can't find the find the right photocell for given state of
		//track return -1 as continued travel is impossible

		else
		{
			p  = p->link; 
			//not in the first node go to the second node

			if (p->switchdirection == Get_TO_Status(p->turnout))
					return ( p->photocell );		
			//if the tostatus matches return this nodes photocell value
			else return  -1;
		}
}



//////////////////////////////////////////////////////////////////////////////
//Returns the next PC assuming that the next Turnout is in the state of the inputed
//switchdirection Used if BatchProcessing is going on and a user wants to check to see
//what the next pc is without actually switching the state of a turnout because that 
//is slow
/////////////////////////////////////////////////////////////////////////////////
int TrackList::FindPCNext(int switchdirection)
{

		Node * p;

	   	p = top->link; // get top of list to start to search

		if (p->turnout == -1)
				return (p->photocell);
		//if no turnout is connected to the photocell then only one
		//photocell can be connected in either direction so just return
		//the photocell stated in the first node

		else if (p->switchdirection == switchdirection)
				return(p->photocell);
	    //if the inputed switchdirection of the turnout matches this one 
		//then return the photocell in this node
				
		else if (p->link == top)
				return  -1; 
		//if we can't find the find the right photocell for given state of
		//track return -1 as continued travel is impossible

		else
		{
			p  = p->link;
			//not in the first node go to the second node
			if (p->switchdirection == switchdirection)
					return ( p->photocell );
			//if the switchdirection matches return this nodes photocell value
				
			else return  -1;
				
		}

}


//////////////////////////////////////////////////////////////////////////////
//Returns the Turnout related to this TrackList
//////////////////////////////////////////////////////////////////////////////
int TrackList::FindToNext(){

		Node * p;

	   	p = top->link;

		return p->turnout;  //return the turnout expressed in this node

}



/////////////////////////////////////////////////////////////////////////
//Finds the necessary turnout diretion to find to get to the inputed photocell
//value
////////////////////////////////////////////////////////////////////////////
int TrackList::FindSwitchDirection(int pc)
{

		Node * p;

	   	p = top->link; // get top of list to start to search
		
		if (p->turnout == -1 && p->photocell == pc)
				return (p->switchdirection);
	
		else if (p->photocell == pc)
				return(p->switchdirection);
					
		else if (p->link == top)
				return 0; 
	
		else
		{
			p  = p->link; 
			//not in the first node go to the second node

			if (p->photocell == pc)
					return (p->switchdirection);		
			else return  0;
		}


}



/*


void main(){


	Turnouts[0]	= STRAIGHT;
	Turnouts[1] = STRAIGHT;
	Turnouts[2]	= STRAIGHT;
	Turnouts[3] = CURVED;
	Turnouts[4] = CURVED;
	Turnouts[5] = CURVED;
	Turnouts[6] = CURVED;

	TrackList CW[MAXPHOTOCELLS];
	TrackList CC[MAXPHOTOCELLS];

	CW[0].Add(2,1,STRAIGHT);
	CW[1].Add(2,1,CURVED);
	CW[2].Add(7,-1,NOTURNOUT);
	CW[3].Add(1,0,STRAIGHT);
	CW[4].Add(12,6,STRAIGHT);
	CW[4].Add(12,6,CURVED);
	CW[5].Add(4,-1,NOTURNOUT);
//	CW[6].Add(12,-1,NOTURNOUT);
	CW[7].Add(15,2,STRAIGHT);
	CW[8].Add(0,-1,NOTURNOUT);
	CW[9].Add(3,-1,NOTURNOUT);
//	CW[10].Add(-1,-1,NOTURNOUT);
	CW[11].Add(5,6,CURVED);
	CW[11].Add(5,6,STRAIGHT);
	CW[12].Add(15,2,CURVED);
	CW[13].Add(8,4,STRAIGHT);
	CW[13].Add(9,4,CURVED);
	CW[14].Add(11,-1,NOTURNOUT);
	CW[15].Add(13,3,STRAIGHT);
	CW[15].Add(14,3,CURVED);


	CC[0].Add(8,-1,NOTURNOUT);
	CC[1].Add(3,0,STRAIGHT);
	CC[2].Add(1,1,CURVED);
	CC[2].Add(0,1,STRAIGHT);
	CC[3].Add(9,-1,NOTURNOUT);
	CC[4].Add(5,-1,NOTURNOUT);
	CC[5].Add(11,6,STRAIGHT);
	CC[5].Add(11,6,CURVED);
//	CC[6].Add(5,-1,NOTURNOUT);
	CC[7].Add(2,-1,STRAIGHT);
	CC[8].Add(13,4,STRAIGHT);
	CC[9].Add(13,4,CURVED);
//	CC[10].Add(-1,-1,-1,NOTURNOUT);
	CC[11].Add(14,-1,NOTURNOUT);
	CC[12].Add(4,6,STRAIGHT);
	CC[12].Add(4,6,CURVED);
	CC[13].Add(15,3,STRAIGHT);
	CC[14].Add(15,3,CURVED);
	CC[15].Add(7,2,STRAIGHT);
	CC[15].Add(12,2,CURVED);



	cout << "With State" << endl;

	for (int i = 0; i< 16; i++){
		cout << "Clockwise of " << i << " is :" <<  CW[i].FindPCNext() << " ";
		cout << "Counter Clockwise of " << i << " is: " << CC[i].FindPCNext() << endl;
		getch();
	}


	cout << endl << "Staight" << endl;

	for (i = 0; i< 16; i++){
		cout << "Clockwise of " << i << " is :" <<  CW[i].FindPCNext(STRAIGHT) << " ";
		cout << "Counter Clockwise of " << i << " is: " << CC[i].FindPCNext(STRAIGHT) << endl;
		getch();
	}

	cout << endl << "Curved" << endl;

	for (i = 0; i< 16; i++){
		cout << "Clockwise of " << i << " is :" <<  CW[i].FindPCNext(CURVED) << " ";
		cout << "Counter Clockwise of " << i << " is: " << CC[i].FindPCNext(CURVED) << endl;
		getch();
	}

	cout << endl << "Moving" << endl;

	for (i = 0; i< 16; i++){
		cout << "Clockwise of " << i << " is :" <<  CW[i].FindPCNext(MOVING) << " ";
		cout << "Counter Clockwise of " << i << " is: " << CC[i].FindPCNext(STRAIGHT) << endl;
		getch();
	}

	cout << endl << "Next TO" <<endl;

	for (i = 0; i< 16; i++){
		cout << "Clockwise of " << i << " is :" <<  CW[i].FindToNext() << " ";
		cout << "Counter Clockwise of " << i << " is: " << CC[i].FindToNext() << endl;
		getch();
	}
	





}

  */

