#include "train.h"
#include <fstream>
#include <string>
#include <afxwin.h>
#include "DCC_comm.h"
using namespace std;


/////////////////////////////////////////////////////////////////////////////
//Constructs a Train-- Needs an id, name, the photocell that the Train Has to start off
//with (ie the one that it already passed) The photocell that it will hit next
//The turnout that it has.  
//Defaulted mode = INTERACTIVE, MaxSpeed = 16, DirectionAroundTrack = ClockWise, 
//Direction = Forward
///////////////////////////////////////////////////////////////////////////////
Train::Train(int Id1, char Name1 [], int PhotocellHas1, int PhotocellNext1, 
			 int TurnoutHas1, int Mode1,  char InputFilename1[], int MaxSpeed1, 
			 int DirectionAroundTrack1, int Direction1)
{
	Id = Id1; // Set id of train
	strcpy(Name, Name1);
	PhotocellHas = PhotocellHas1; // the photocells bounding the track it starts on
	
	//************************************************************
	//Added: Spring 2002
	// initialize PCHas as the PC last successfully departed from
	DepartedPC = PhotocellHas1; 
	//************************************************************

	PhotocellNext = PhotocellNext1;
	TurnoutHas = TurnoutHas1; // and the turnout if there is one
	strcpy(InputFilename, InputFilename1);

	//init rest of trains variables
	DirectionAroundTrack = DirectionAroundTrack1;
	Direction = Direction1;
	Mode = Mode1;
	MaxSpeed = MaxSpeed1;


	BlockingPhotocell = -1;
	BlockingTurnout = -1;		//bmcvey 3-25-02  JV had not set
	OldPhotocellHas = -1;		//bmcvey 3-25-02  JV had set to 0
	BlockInstructions = 1;
	OldTurnoutHas = -1;			//bmcvey 3-25-02  JV had set to 0
	HeadLight = OFF;
	TailLight = OFF;
	LastPhotocellCovered = -1;
	LastUserSpeed = 0;
	DCCLastCommand = "";
	Speed = 0;



	ifstream fin;

	//open and read the Input filename if the it is not a null file
	//into an array of lines.  Had trouble making copies of the 
	//ifstream handlers
	LinesRead = 0;
	if (strcmp(InputFilename, ""))
	{
		fin.open(InputFilename);

		getline(fin, Input[LinesRead]);

		while (!fin.eof())
		{
			LinesRead++;
			getline(fin, Input[LinesRead]);
		}

	}

	if (strcmp(InputFilename, ""))
		fin.close();

	cursor = 0; //used to loop for reading

	if (Mode == BATCH) //init speed until it hits its first photocell
		SetSpeed(6);
	
}



Train::~Train()
{

}


////////////////////////////////////////////////////////////////////////
//Copy Constructor Makes a copy of a Train
//////////////////////////////////////////////////////////////////////////
Train::Train(const Train & T){


	 Id = T.Id;
	 Speed = T.Speed;
	 MaxSpeed = T.MaxSpeed;
	 OldPhotocellHas = T.OldPhotocellHas;
	 PhotocellHas = T.PhotocellHas;

	 //************************************************************
	 //Added: Spring 2002
	 // initialize PCHas as the PC last successfully departed from
	 DepartedPC = T.PhotocellHas;
	 //************************************************************

	 PhotocellNext = T.PhotocellNext;
	 TurnoutHas = T.TurnoutHas;
	 OldTurnoutHas = T.OldTurnoutHas;
	 LastPhotocellCovered = T.LastPhotocellCovered;
	 DirectionAroundTrack = T.DirectionAroundTrack;
	 Direction = T.Direction;
	 HeadLight = T.HeadLight;
	 TailLight = T.TailLight;
	 Mode = T.Mode;
	 BlockInstructions = T.BlockInstructions;
	 BlockingPhotocell = T.BlockingPhotocell;
	 strcpy(Name, T.Name);
	 LastUserSpeed = T.LastUserSpeed;
	 strcpy(InputFilename, T.InputFilename);
	 DCCLastCommand = T.DCCLastCommand;
	 for (int i = 0; i<MAXINPUTLINES ; i++)
		 Input[i] = T.Input[i];
	

}



/////////////////////////////////////////////////////////////////////
//Sets the Speed of a train to be specified speed Calls Increase Speed
//or DecreaseSpeed to adjust speed as necessary returns the value of 
//IncreaseSpeed or DecreaseSpeed return 1 if speed did not change
////////////////////////////////////////////////////////////////////////
int Train::SetSpeed(int newspeed )//, HANDLE DDC_hcomm)
{

	

	if (MyDCC_motor(Id, Direction, newspeed))
	{
		Speed = newspeed;

		return 1;

	}

	else return 0;
	
}





////////////////////////////////////////////////////////////////
//Stops a train by setting its speed to 0
///////////////////////////////////////////////////////////////
int Train::Stop()
{
	return (SetSpeed(0));
}



/////////////////////////////////////////////////////////////////////////
//Toggles the HeadLight if inputed Parameter does not match the current state
//of the Headlight  Sends fuction command to systemone box
//////////////////////////////////////////////////////////////////////////
int Train::TurnHeadLightOn(int TurnOn)
{


	if (TurnOn != HeadLight)
		return (ToggleHeadLight());
	else
		return 1;

}

/////////////////////////////////////////////////////////////////////////
//Toggles the TailLight if inputed Parameter does not match the current state
//of the Headlight send function command to systemone box
//////////////////////////////////////////////////////////////////////////
int Train::TurnTailLightOn(int TurnOn)
{


	if (TurnOn != TailLight)
		return (ToggleTailLight());
	else 
		return 1;


}


///////////////////////////////////////////////////////////////////
//Toggles the Current State of the HeadLight and send the command to the
//systemOne board
//////////////////////////////////////////////////////////////////////
int Train::ToggleHeadLight()
{
 

   if(HeadLight == ON)
   {  
      HeadLight = OFF; //Turn off
   }
   else 
   {  
      HeadLight = ON; //Turn on
   }

   return 1;

}


int Train::ToggleTailLight()
///////////////////////////////////////////////////////////////////
//Toggles the Current State of the TailLight and send the command to the
//systemOne board
//////////////////////////////////////////////////////////////////////
{


   if(TailLight == ON)//Taillight is on
   { 
      TailLight = OFF; //Turn off
   }
   else 
   {
      TailLight = ON; //Turn on
   }

   return 1;

}

///////////////////////////////////////////////////////////////////////////////
//Changes the Direction of a train.  Remembers the oldSpeed Stops the Train toggles the
//DirectionAroundTrack. Toggles the Train's direction
//Swaps the PhotocellHas and PhotoCellNext and starts the train again with 
//remembered old speed
//////////////////////////////////////////////////////////////////////////////////
int Train::ChangeDirection(){

   
   int oldspeed;

   oldspeed = Speed;


   Stop();
  
   if(Direction == FORWARD)
   {
	 Direction = REVERSE; //Change from forward to reverse
   }
   else
   {
	 Direction = FORWARD; //Change from reverse to forward
   }

   if (DirectionAroundTrack == COUNTERCLOCKWISE) //Change from counter-
   {
	   DirectionAroundTrack = CLOCKWISE;
   }
   else //Change from clockwise to counterclockwise
   {
	   DirectionAroundTrack = COUNTERCLOCKWISE;
   }

   swap(PhotocellHas, PhotocellNext);

   if (SetSpeed(oldspeed))
		return 1; //Direction not changed 
   else
	   return 0;

}

/////////////////////////////////////////////////////
//Sets speed to the last User supplied Speed
//////////////////////////////////////////////////////
int Train::ExecuteLastDCC_Command()
{

	return (SetSpeed(LastUserSpeed));

	

}

/////////////////////////////////////////////////////////////////////
//Reads the next photocell for a Train running on batch mode
//note that all files end with 9999 which represents a loop which begins
//at the first photocell again.  This closes, opens and begins reading 
//from the file again
////////////////////////////////////////////////////////////////////////
int Train::ReadInstruction(int & pc, int &dir, string& command)
{
	string line = Input[cursor];

	pc = atoi(line.substr(0, 2).c_str());
	dir = atoi(line.substr(2, 4).c_str());
	command = line.substr(6, line.size());

	if(cursor == LinesRead)
		cursor = 0;
	else
		cursor++;

	return 1;

}
///////////////////////////////////////////////////////////////////////
//Returns the PhotoCell the Train Has(ie the photocell it already covered
//bounding the track that it is on
/////////////////////////////////////////////////////////////////////////
int Train::GetPhotocellHas(){

	return PhotocellHas;


}

///////////////////////////////////////////////////////////////////////
//Returns the PhotoCell the Train will have next(ie the photocell it did not covered
//bounding the track that it is on
/////////////////////////////////////////////////////////////////////////

int Train::GetPhotocellNext()
{
	return PhotocellNext;
}


///////////////////////////////////////////////////////////////////////
//Returns the Turnout the Train Has(ie the Turnout it has on the track that it is on
/////////////////////////////////////////////////////////////////////////
int Train::GetTurnoutHas()
{
	return TurnoutHas;
}




/////////////////////////////////////////////////////////////////
//Returns Photocell that is blocking the Train
////////////////////////////////////////////////////////////////
int Train::GetBlockingPhotocell()
{


	return BlockingPhotocell;

}




/////////////////////////////////////////////////////////////////////////
//Returns the last photocell covered
/////////////////////////////////////////////////////////////////////////
int Train::GetLastPhotocellCovered()
{

	return LastPhotocellCovered;

}


//////////////////////////////////////////////////////////////////////////
//Returns the Train's DirectionAroundTrack ie (Clockwise or Counterclockwise
////////////////////////////////////////////////////////////////////////////
int Train::GetDirectionAroundTrack()
{

	return DirectionAroundTrack;

}



////////////////////////////////////////////////////////////////////////////
//Returns the Trains Direction ie Forward or Reverse
////////////////////////////////////////////////////////////////////////////
int Train::GetDirection()
{

	return Direction;

}



//////////////////////////////////////////////////////////////////////
//Returns the id of a train
////////////////////////////////////////////////////////////////////
int Train::GetId()
{

	return Id; //return id

}


/////////////////////////////////////////////////////////
//Returns name of Train
////////////////////////////////////////////////////////
char * Train::GetName()
{
	return Name;

}


//////////////////////////////////////////////////////////////////////
//returns the speed of a train
////////////////////////////////////////////////////////////////////	
int Train::GetSpeed()
{

	return Speed;//return speed
}

//////////////////////////////////////////////////////////
//Return ON or OFF for current State of Headlight
///////////////////////////////////////////////////////////
int Train::GetHeadLight()
{
	return HeadLight;

}


//////////////////////////////////////////////////////////
//Return ON or OFF for current State of Taillight
///////////////////////////////////////////////////////////
int Train::GetTailLight()
{

	return TailLight;

}

//////////////////////////////////////////////////
//Returns Active or Blocked for Status of Train
//////////////////////////////////////////////////
int Train::GetBlockedStatus()
{

	return BlockedStatus;

}

////////////////////////////////////////////////
//Returns Trains MaxSpeed
/////////////////////////////////////////////////
int Train::GetMaxSpeed()
{

	return MaxSpeed;


}
////////////////////////////////////////////////////////////////////////////
//Returns the Mode ie INTERACTIVE OR BATCH of the train
///////////////////////////////////////////////////////////////////////////
int Train::GetMode(){

	return Mode;

}


/////////////////////////////////////////////////////////////////////////////
//Swaps two integers
////////////////////////////////////////////////////////////////////////////
void swap(int & x, int & y){

	int temp = x;
	x = y;
	y = temp;

}



/*



void main(){


	Train X(1, 16);

	X.Print();

	getch();

	X.IncreaseSpeed(2);

	X.Print();

	getch();

	X.DecreaseSpeed();

	X.Print();

	getch();

	X.ChangeDirection();

	X.Print();
}




*/

